@@ -20,6 +20,11 @@
  ***************************************************************************/
 
 
+#ifdef MSVC_QMAKE
+    #pragma warning (disable : 4100)
+#endif
+
+
 #include  <cmath>
 #include  <algorithm>
 #include  <sstream>
@@ -46,6 +51,7 @@
 #include  "StreamsModel.h"    // current streams
 #include  "UniqueNotesModel.h"   // all notes
 #include  "CommonTypes.h"
+#include  "Widgets.h"
 
 using namespace std;
 using namespace pearl;
@@ -196,6 +202,7 @@ void SessionSettings::saveMiscConfigSettings(const CommonData* p)
     }
 
     { // misc
+        m_pSettings->setValue("main/showExport", p->m_bShowExport);
         m_pSettings->setValue("main/showDebug", p->m_bShowDebug);
         m_pSettings->setValue("main/showSessions", p->m_bShowSessions);
         m_pSettings->setValue("normalizer/command", convStr(p->m_strNormalizeCmd));
@@ -207,6 +214,8 @@ void SessionSettings::saveMiscConfigSettings(const CommonData* p)
         m_pSettings->setValue("debug/saveDownloadedData", p->m_bSaveDownloadedData);
         m_pSettings->setValue("main/autoSizeIcons", p->m_bAutoSizeIcons);
         m_pSettings->setValue("main/keepOneValidImg", p->m_bKeepOneValidImg);
+        m_pSettings->setValue("main/processWmpVarArtists", p->m_bWmpVarArtists);
+        m_pSettings->setValue("main/processItunesVarArtists", p->m_bItunesVarArtists);
         m_pSettings->setValue("main/fastSave", p->useFastSave());
         m_pSettings->setValue("debug/traceToFile", p->isTraceToFileEnabled());
 
@@ -220,6 +229,15 @@ void SessionSettings::saveMiscConfigSettings(const CommonData* p)
 
         m_pSettings->setValue("main/warnedAboutSel", p->m_bWarnedAboutSel);
         m_pSettings->setValue("main/warnedAboutBackup", p->m_bWarnedAboutBackup);
+        m_pSettings->setValue("tagEditor/toldAboutPatterns", p->m_bToldAboutPatterns);
+        m_pSettings->setValue("main/toldAboutSupport", p->m_bToldAboutSupport);
+
+        m_pSettings->setValue("fileRenamer/invalidChars", convStr(p->m_strRenamerInvalidChars));
+        m_pSettings->setValue("fileRenamer/replacementForInvalid", convStr(p->m_strRenamerReplacementString));
+
+        m_pSettings->setValue("main/checkForNewVersions", convStr(p->m_strCheckForNewVersions));
+        m_pSettings->setValue("main/timeLastNewVerCheck", p->m_timeLastNewVerCheck);
+        m_pSettings->setValue("main/dontTellAboutVer", convStr(p->m_strDontTellAboutVer));
     }
 
     { // note categ colors
@@ -276,15 +294,20 @@ void SessionSettings::loadMiscConfigSettings(CommonData* p) const
         p->m_pCodec = (QTextCodec::codecForName(p->m_locale));
         if (0 == p->m_pCodec)
         {
-            QList<QByteArray> l (QTextCodec::availableCodecs());
+            /*QList<QByteArray> l (QTextCodec::availableCodecs());
             CB_ASSERT (l.size() > 0);
-            p->m_locale = l.front();
+            p->m_locale = l.front();*/
+            p->m_locale = "System"; // ttt2 2009.11.03 - actually this seems a bad idea; at least on 10.3 / 4.3.1 setting the locale to System shows no characters for codes > 128; doesn't happen in W7, though
             p->m_pCodec = (QTextCodec::codecForName(p->m_locale));
         }
+        else
+        {
+            p->m_locale = p->m_pCodec->name(); // !!! needed because the names may be different ("ISO 8859-1" vs. "ISO-8859-1")
+        }
         CB_ASSERT (0 != p->m_pCodec);
 
-        p->m_eCaseForArtists = (CommonData::Case)m_pSettings->value("id3V2Transf/caseForArtists", 2).toInt();
-        p->m_eCaseForOthers = (CommonData::Case)m_pSettings->value("id3V2Transf/caseForOthers", 3).toInt();
+        p->m_eCaseForArtists = (TextCaseOptions)m_pSettings->value("id3V2Transf/caseForArtists", 2).toInt();
+        p->m_eCaseForOthers = (TextCaseOptions)m_pSettings->value("id3V2Transf/caseForOthers", 3).toInt();
     }
 
     { // tag editor
@@ -296,6 +319,7 @@ void SessionSettings::loadMiscConfigSettings(CommonData* p) const
     }
 
     { // misc
+        p->m_bShowExport = m_pSettings->value("main/showExport", false).toBool();
         p->m_bShowDebug = m_pSettings->value("main/showDebug", false).toBool();
         p->m_bShowSessions = m_pSettings->value("main/showSessions", !p->isUniqueSession()).toBool();
         p->m_strNormalizeCmd = convStr(m_pSettings->value("normalizer/command", "mp3gain -a -k -p -t").toString());
@@ -307,6 +331,8 @@ void SessionSettings::loadMiscConfigSettings(CommonData* p) const
         p->m_bSaveDownloadedData = m_pSettings->value("debug/saveDownloadedData", false).toBool();
         p->m_bAutoSizeIcons = m_pSettings->value("main/autoSizeIcons", true).toBool();
         p->m_bKeepOneValidImg = m_pSettings->value("main/keepOneValidImg", false).toBool();
+        p->m_bWmpVarArtists = m_pSettings->value("main/processWmpVarArtists", false).toBool();
+        p->m_bItunesVarArtists = m_pSettings->value("main/processItunesVarArtists", false).toBool();
         p->setFastSave(m_pSettings->value("main/fastSave", false).toBool(), CommonData::DONT_UPDATE_TRANSFORMS);
         p->setTraceToFile(m_pSettings->value("debug/traceToFile", false).toBool());
 
@@ -318,6 +344,25 @@ void SessionSettings::loadMiscConfigSettings(CommonData* p) const
 
         p->m_bWarnedAboutSel = m_pSettings->value("main/warnedAboutSel", false).toBool();
         p->m_bWarnedAboutBackup = m_pSettings->value("main/warnedAboutBackup", false).toBool();
+        p->m_bToldAboutPatterns = m_pSettings->value("tagEditor/toldAboutPatterns", false).toBool();
+        p->m_bToldAboutSupport = m_pSettings->value("main/toldAboutSupport", false).toBool();
+
+#ifndef WIN32
+        const char* DEFAULT_INVALID ("/\"\\*?<>|"); //ttt2 even in Unix, it might be a good idea to not allow ":" as well in some cases, depending on the file system
+#else
+        const char* DEFAULT_INVALID ("/\"\\*?<>|:"); //ttt2 perhaps have yellow background instead of underline, to be able to tell if there's a space or not; or have something like a progres bar below; or have each letter in its own rectangle;
+#endif
+        p->m_strRenamerInvalidChars = convStr(m_pSettings->value("fileRenamer/invalidChars", DEFAULT_INVALID).toString());
+        p->m_strRenamerReplacementString = convStr(m_pSettings->value("fileRenamer/replacementForInvalid", "_").toString());
+
+        p->m_strCheckForNewVersions = convStr(m_pSettings->value("main/checkForNewVersions", "").toString());
+        p->m_timeLastNewVerCheck = m_pSettings->value("main/timeLastNewVerCheck").toDateTime();
+        if (p->m_timeLastNewVerCheck.isNull())
+        {
+            QDateTime t1 (QDateTime::currentDateTime());
+            p->m_timeLastNewVerCheck = t1.addYears(-1);
+        }
+        p->m_strDontTellAboutVer = convStr(m_pSettings->value("main/dontTellAboutVer", "").toString());
     }
 
     { // note categ colors
@@ -428,7 +473,7 @@ void CommonData::setFontInfo(const std::string& strGenName, int nGenSize, int nL
 
     if (!bFirstTime)
     {
-        QMessageBox::warning(m_pFilesG, "Info", "The font changes will only be used after restarting the application."); //ttt1 try to get this work, probably needs to call QHeaderView::resizeSection(), as well as review all setMinimumSectionSize() and setDefaultSectionSize() calls;
+        QMessageBox::warning(m_pFilesG, "Info", "The font changes will only be used after restarting the application."); //ttt2 try to get this work, probably needs to call QHeaderView::resizeSection(), as well as review all setMinimumSectionSize() and setDefaultSectionSize() calls;
         return;
     }
 
@@ -447,23 +492,12 @@ void CommonData::setFontInfo(const std::string& strGenName, int nGenSize, int nL
     //m_fixedFont.setFixedPitch(true);  // !!! needed to select some fixed font in case there's no "B&H LucidaTypewriter" installed
     m_fixedFont.setFixedPitch(true);
     m_fixedFont.setStyleHint(QFont::Courier);
-    //f.setPixelSize(12); //ttt2 hard-coded "12"; //ttt1 some back-up font instead of just letting the system decide
+    //f.setPixelSize(12); //ttt2 hard-coded "12"; //ttt2 some back-up font instead of just letting the system decide
     m_fixedFont.setPointSize(nFixedSize);
 
 
     CELL_HEIGHT = QApplication::fontMetrics().height() + 3;
 
-    //qDebug("wdth %d", QApplication::fontMetrics().width("W"));
-    //qDebug("wdth %d", QApplication::fontMetrics().maxWidth()); // too big, probably from non-ASCII-letter chars
-    /*int n (QApplication::fontMetrics().width("W")); // ttt1 switch to W', M' and m' when needed
-    n = max(n, QApplication::fontMetrics().width("M"));
-    n = max(n, QApplication::fontMetrics().width("m"));
-#ifndef WIN32
-    CELL_WIDTH = n + 5;
-#else
-    CELL_WIDTH = n + 6; //ttt2 hard-coded "6"; see how to get the correct value
-#endif*/
-
     {
         QPixmap img (100, 100); //ttt2 revisit the size; might need increase in the future
         QPainter pntr (&img);
@@ -486,7 +520,7 @@ void CommonData::setFontInfo(const std::string& strGenName, int nGenSize, int nL
                     n = max(n, r.width());
                 }
             }
-            //qDebug("%s - %d %d %2d %2d", s.toUtf8().data(), r.x(), r.y(), r.width(), r.height());
+            //qDebug("%s - %d %d %2d %2d", s.toUtf8().constData(), r.x(), r.y(), r.width(), r.height());
         }
 
         CELL_WIDTH = n + 4; //ttt2 hard-coded "4"; see how to get the correct value
@@ -528,6 +562,16 @@ QFont CommonData::getNewFixedFont() const
 //=====================================================================================================================
 
 
+namespace
+{
+    CommonData* g_pCommonData (0);
+}
+
+CommonData* getCommonData() // ttt2 get rid of all places passing CommonData* as a param
+{
+    return g_pCommonData;
+}
+
 
 CommonData::CommonData(
         SessionSettings& settings,
@@ -565,6 +609,8 @@ CommonData::CommonData(
         m_nMainWndIconSize(40),
         m_settings(settings),
         m_bKeepOneValidImg(false),
+        m_bWmpVarArtists(false),
+        m_bItunesVarArtists(false),
         m_bLogTransf(false),
         m_bSaveDownloadedData(false),
         m_vvnCustomTransf(CUSTOM_TRANSF_CNT),
@@ -581,6 +627,8 @@ CommonData::CommonData(
         m_pModeAlbumB(pModeAlbumB),
         m_pModeSongB(pModeSongB)
 {
+    g_pCommonData = this;
+
     m_vpAllTransf.push_back(new SingleBitRepairer());
     m_vpAllTransf.push_back(new InnerNonAudioRemover());
 
@@ -641,6 +689,7 @@ CommonData::~CommonData()
 {
     clearPtrContainer(m_vpAllHandlers);
     clearPtrContainer(m_vpAllTransf);
+    g_pCommonData = 0;
 }
 
 
@@ -948,6 +997,7 @@ int CommonData::getPosInFlt(const Mp3Handler* pMp3Handler) const
 
 void CommonData::onCrtFileChanged()
 {
+    CursorOverrider crs;
     updateCurrentNotes();
     updateCurrentStreams();
 }
@@ -1150,9 +1200,9 @@ set<string> CommonData::getAllDirs() const
     {
         const Mp3Handler* p (m_vpAllHandlers[i]);
         const string& s (p->getName());
-        string::size_type n (s.rfind(getPathSep()));
-        CB_ASSERT(string::npos != n);
-        string strDir (s.substr(0, n));
+        string::size_type k (s.rfind(getPathSep()));
+        CB_ASSERT(string::npos != k);
+        string strDir (s.substr(0, k));
         sDirs.insert(strDir);
     }
 
@@ -1346,7 +1396,7 @@ struct TestLabel
     {
         for (int i = 0; i < 104; ++i)
         {
-            qDebug("%d %s", i, getNoteLabel(i).toUtf8().data());
+            qDebug("%d %s", i, getNoteLabel(i).toUtf8().constData());
         }
     }
 };
@@ -1379,6 +1429,7 @@ const QColor& SUPPORT_PEN_COLOR()
 // color is normally the category color, but for support notes it's a "support" color; if the note isn't found in vpNoteSet, dGradStart and dGradEnd are set to -1, but normally they get a segment obtained by dividing [0, 1] in equal parts;
 void CommonData::getNoteColor(const Note& note, const vector<const Note*>& vpNoteSet, QColor& color, double& dGradStart, double& dGradEnd) const
 {
+    LAST_STEP("CommonData::getNoteColor()");
     dGradStart = -1;
     dGradEnd = -1;
 
@@ -1492,10 +1543,10 @@ const QFont& getFixedFont()
         s_font.setFamily("B&H LucidaTypewriter");
         //s_font.setFamily("B&H LucidaTypewriter12c");
         s_font.setFixedPitch(true);  // !!! needed to select some fixed font in case there's no "B&H LucidaTypewriter" installed
-        s_font.setPixelSize(12); //ttt2 hard-coded "12"; //ttt1 some back-up font instead of just letting the system decide
-        //s_font.setPointSize(9); //ttt2 hard-coded "9"; //ttt2 some back-up font instead of just letting the system decide
+        s_font.setPixelSize(12); //ttt2 hard-coded "12";
+        //s_font.setPointSize(9); //ttt2 hard-coded "9";
 
-//QFont f; qDebug("default font: pix %d, point: %d, fam: %s", f.pixelSize(), f.pointSize(), f.family().toLatin1().data());
+//QFont f; qDebug("default font: pix %d, point: %d, fam: %s", f.pixelSize(), f.pointSize(), f.family().toLatin1().constData());
 
         //myOption.font.setPointSize(12);
 //        printFontInfo("new font: ", s_font);
@@ -1592,15 +1643,15 @@ void Filter::setNotes(const std::vector<const Note*>& v)
 
 void Filter::setDirs(const std::vector<string>& v)
 {
-    //if (v == m_vDirs) { return; }
-    m_vDirs = v;
-    m_bDirFilter = !m_vDirs.empty();
+    //if (v == m_vstrDirs) { return; }
+    m_vstrDirs = v;
+    m_bDirFilter = !m_vstrDirs.empty();
 
     emit filterChanged();
 }
 
 
-// !!! needed because we need m_vDirs next time we press the filter button, so we don't want to delete it with a "setDirs(vector<string>())", but just ignore it
+// !!! needed because we need m_vstrDirs next time we press the filter button, so we don't want to delete it with a "setDirs(vector<string>())", but just ignore it
 void Filter::disableDir()
 {
     if (!m_bDirFilter) { return; }
@@ -1654,12 +1705,14 @@ void Filter::restoreAll() // loads m_bNoteFilter from m_bSavedNoteFilter and m_b
 // this is needed after transforms/normalization/tag editing, but there's no need for an explicit call, because all these call mergeHandlerChanges() (directly or through MainFormDlgImpl::scan())
 void CommonData::updateWidgets(const std::string& strCrtName /*= ""*/, const std::vector<std::string>& vstrSel /*= std::vector<std::string>()*/)
 {
-    if (m_vpViewHandlers.empty() && !m_vpFltHandlers.empty())
+    CursorOverrider crs;
+
+    //if (m_vpViewHandlers.empty() && !m_vpFltHandlers.empty()) // 2009.09.25 - not sure why the test was here, but its presence prevented the right song to be selected when returning from tag editing, if in album mode; see if this takes a long time; if so, perhaps the idea was to run this only if needed, but the test was too restrictive
     {
         CommonData::ViewMode eViewMode (getViewMode());
         setViewMode(CommonData::ALL);
         int nPos (getPosInView(strCrtName)); // doesn't matter if it's empty
-        const Mp3Handler* pMp3Handler (m_vpViewHandlers[nPos]);
+        const Mp3Handler* pMp3Handler (nPos >= 0 ? m_vpViewHandlers.at(nPos) : 0);
         setViewMode(eViewMode, pMp3Handler);
     }
 
@@ -1832,6 +1885,7 @@ const Mp3Handler* CommonData::getHandler(const std::string& strName) const
 void CommonData::mergeHandlerChanges(const vector<const Mp3Handler*>& vpAdd1, const vector<const Mp3Handler*>& vpDel1, int nKeepWhenUpdate)
 {
     //m_bDirty = m_bDirty || !vpAdd1.empty() || !vpDel1.empty();
+    string strSongInCrtAlbum (m_nSongInCrtAlbum >= 0 && m_nSongInCrtAlbum < cSize(m_vpAllHandlers) ? m_vpAllHandlers[m_nSongInCrtAlbum]->getName() : "");
 
     const string& strCrtName (nKeepWhenUpdate & CURRENT ? getCrtName() : "");
     const vector<string>& vstrSel (nKeepWhenUpdate & SEL ? getSelNames() : vector<string>());
@@ -1872,15 +1926,24 @@ qDebug("1c %d %d %d", cSize(vpAll), cSize(vpFlt), cSize(vpView));*/
     set_difference(m_vpAllHandlers.begin(), m_vpAllHandlers.end(), vpDel.begin(), vpDel.end(), back_inserter(vpAll), CmpMp3HandlerPtrByName());
     set_difference(m_vpFltHandlers.begin(), m_vpFltHandlers.end(), vpDel.begin(), vpDel.end(), back_inserter(vpFlt), CmpMp3HandlerPtrByName());
     set_difference(m_vpViewHandlers.begin(), m_vpViewHandlers.end(), vpDel.begin(), vpDel.end(), back_inserter(vpView), CmpMp3HandlerPtrByName());
+//qDebug("m_vpAllHandlers.z()=%d, m_vpFltHandlers.size()=%d, m_nSongInCrtAlbum=%d, vpDel.sz=%d, vpDel1.sz=%d, vpAdd1.sz=%d", cSize(m_vpAllHandlers), cSize(m_vpFltHandlers), m_nSongInCrtAlbum, cSize(vpDel), cSize(vpDel1), cSize(vpAdd1));
 
     clearPtrContainer(vpDel);
 
     m_vpAllHandlers.clear(); m_vpFltHandlers.clear(); m_vpViewHandlers.clear();
     set_union(vpAll.begin(), vpAll.end(), vpAdd.begin(), vpAdd.end(), back_inserter(m_vpAllHandlers), CmpMp3HandlerPtrByName());
-    set_union(vpFlt.begin(), vpFlt.end(), vpAdd.begin(), vpAdd.end(), back_inserter(m_vpFltHandlers), CmpMp3HandlerPtrByName()); // !!! perhaps for view and flt we should check the directory/filter, but it seems more important that the new handlers are seen, so they can be compared to the old ones //ttt1 review
+    set_union(vpFlt.begin(), vpFlt.end(), vpAdd.begin(), vpAdd.end(), back_inserter(m_vpFltHandlers), CmpMp3HandlerPtrByName()); // !!! perhaps for view and flt we should check the directory/filter, but it seems more important that the new handlers are seen, so they can be compared to the old ones //ttt2 review
     set_union(vpView.begin(), vpView.end(), vpAdd.begin(), vpAdd.end(), back_inserter(m_vpViewHandlers), CmpMp3HandlerPtrByName());
 
     updateWidgets(strCrtName, vstrSel);
+
+    if (!strSongInCrtAlbum.empty() && !m_vpFltHandlers.empty())
+    {
+        deque<const Mp3Handler*>::const_iterator it (lower_bound(m_vpFltHandlers.begin(), m_vpFltHandlers.end(), strSongInCrtAlbum, CmpMp3HandlerPtrByName()));
+        if (m_vpFltHandlers.end() == it) { --it; }
+        it = lower_bound(m_vpAllHandlers.begin(), m_vpAllHandlers.end(), *it, CmpMp3HandlerPtrByName());
+        m_nSongInCrtAlbum = it - m_vpAllHandlers.begin();
+    }
 }
 
 
@@ -1901,7 +1964,7 @@ void CommonData::setSongInCrtAlbum()
 deque<const Mp3Handler*> CommonData::getCrtAlbum() const //ttt2 perhaps sort by track, if there's an ID3V2; anyway, this is only used by TagWriter and FileRenamer, and TagWriter does its own sorting (there might be an issue if calling getCrtAlbum() multiple times for the same album returns songs in different order because tracks changed between the calls)
 {
     deque<const Mp3Handler*> v;
-    if (m_vpAllHandlers.empty()) { return v; }
+    if (m_vpFltHandlers.empty()) { return v; }
 
     //CB_ASSERT (m_nSongInCrtAlbum >= 0 && m_nSongInCrtAlbum < cSize(m_vpAllHandlers)); // !!! incorrect assert; files migth have been deleted without updating m_nSongInCrtAlbum (well, one of the purposes of this function is to update m_nSongInCrtAlbum)
     CB_ASSERT (m_nSongInCrtAlbum >= 0);
@@ -1923,10 +1986,11 @@ deque<const Mp3Handler*> CommonData::getCrtAlbum() const //ttt2 perhaps sort by
 // used for album navigation in tag editor and file renamer
 bool CommonData::nextAlbum() const
 {
-    if (m_vpAllHandlers.empty()) { return false; }
+    if (m_vpFltHandlers.empty()) { return false; }
 
     CB_ASSERT (m_nSongInCrtAlbum >= 0 && m_nSongInCrtAlbum < cSize(m_vpAllHandlers));
     string s (m_vpAllHandlers[m_nSongInCrtAlbum]->getDir());
+//qDebug("m_vpAllHandlers.z()=%d, m_vpFltHandlers.size()=%d, m_nSongInCrtAlbum=%d", cSize(m_vpAllHandlers), cSize(m_vpFltHandlers), m_nSongInCrtAlbum);
 
     deque<const Mp3Handler*>::const_iterator it (lower_bound(m_vpFltHandlers.begin(), m_vpFltHandlers.end(), m_vpAllHandlers[m_nSongInCrtAlbum], CmpMp3HandlerPtrByName()));
     //CB_ASSERT (m_vpAllHandlers[m_nSongInCrtAlbum] == *it); // !!! wrong - getCrtAlbum() moves m_nSongInCrtAlbum to the first song in album regardless of filter
@@ -1949,10 +2013,9 @@ bool CommonData::nextAlbum() const
 
 
 
-
 bool CommonData::prevAlbum() const
 {
-    if (m_vpAllHandlers.empty()) { return false; }
+    if (m_vpFltHandlers.empty()) { return false; }
 
     CB_ASSERT (m_nSongInCrtAlbum >= 0 && m_nSongInCrtAlbum < cSize(m_vpAllHandlers));
     string s (m_vpAllHandlers[m_nSongInCrtAlbum]->getDir());
@@ -2036,6 +2099,10 @@ void CommonData::setFastSave(bool bFastSave, bool bUpdateTransforms)
 }
 
 
+
+
+
+
 /*
 
 KDE 3.5.7 / Qt 3.3.8
