@@ -28,6 +28,7 @@
 
 //#include  <QColor>
 #include  <QFont>
+#include  <QDateTime>
 
 #include  "SerSupport.h"
 
@@ -35,6 +36,7 @@
 #include  "Mp3Manip.h"
 #include  "Helpers.h"
 #include  "FileEnum.h"
+#include  "CommonTypes.h"
 
 //class QFont;
 
@@ -45,11 +47,11 @@
 
 class QToolButton;
 class FilesModel;
-class NotesModel;
-class StreamsModel;
-class UniqueNotesModel;
+struct NotesModel;
+struct StreamsModel;
+struct UniqueNotesModel;
 
-extern int CELL_WIDTH; // ttt1 perhaps replace with functions
+extern int CELL_WIDTH; // ttt2 perhaps replace with functions
 extern int CELL_HEIGHT;
 
 extern const int CUSTOM_TRANSF_CNT; // search for this to find all places that need changes to add another custom transform list
@@ -163,16 +165,16 @@ class Filter : public QObject
     bool m_bSavedNoteFilter;
     bool m_bSavedDirFilter;
 
-    std::vector<std::string> m_vDirs;
+    std::vector<std::string> m_vstrDirs;
     std::vector<const Note*> m_vpNotes; // doesn't own the pointers, because it is just a subset of CommonData::m_uniqueNotes.getAllVec(), which in turn uses pointers from Notes:: // !!! it's OK for this to be a vector (rather than a set), because the way it is used most of the time, namely to determine if the intersection of 2 sets (represented as sorted vectors) is empty or not
 public:
     Filter() : m_bNoteFilter(false), m_bDirFilter(false), m_bSavedNoteFilter(false), m_bSavedDirFilter(false) {}
 
-    const std::vector<std::string>& getDirs() const { return m_vDirs; }
+    const std::vector<std::string>& getDirs() const { return m_vstrDirs; }
     const std::vector<const Note*>& getNotes() const { return m_vpNotes; }
     void setDirs(const std::vector<std::string>&);
     void setNotes(const std::vector<const Note*>&);
-    void disableDir(); // !!! needed because we need m_vDirs next time we press the filter button, so we don't want to delete it with a "setDirs(vector<string>())", but just ignore it
+    void disableDir(); // !!! needed because we need m_vstrDirs next time we press the filter button, so we don't want to delete it with a "setDirs(vector<string>())", but just ignore it
     void disableNote();
 
     void disableAll(); // saves m_bNoteFilter to m_bSavedNoteFilter and m_bDirFilter to m_bSavedDirFilter, then disables the filters
@@ -196,21 +198,23 @@ private:
         ar & m_bSavedNoteFilter;
         ar & m_bSavedDirFilter;
 
-        ar & m_vDirs;
+        ar & m_vstrDirs;
         ar & m_vpNotes;
     }*/
 
-    template<class Archive> void save(Archive& ar, const unsigned int /*nVersion*/) const
+    template<class Archive> void save(Archive& ar, const unsigned int nVersion) const
     {
+        if (nVersion > 0) { throw std::runtime_error("invalid version of serialized file"); }
+
         ar << m_bNoteFilter;
         ar << m_bDirFilter;
 
         ar << m_bSavedNoteFilter;
         ar << m_bSavedDirFilter;
 
-        ar << m_vDirs;
-        //qDebug("saved dirs sz %d", cSize(m_vDirs));
-        //ar << m_vpNotes; //ttt1 weird behaviour: this compiles and doesn't trigger runtime errros, and neither does the loading, but when loading a vector<Note*> it always ends up empty; it's probably some incorrect use of the ser library (share / global / const pointers), but the library is broken too, because it should have failed to compile or at least crashed when running instead of just failing to load anything (the files are different, so something is saved)
+        ar << m_vstrDirs;
+        //qDebug("saved dirs sz %d", cSize(m_vstrDirs));
+        //ar << m_vpNotes; //ttt2 weird behaviour: this compiles and doesn't trigger runtime errros, and neither does the loading, but when loading a vector<Note*> it always ends up empty; it's probably some incorrect use of the ser library (share / global / const pointers), but the library is broken too, because it should have failed to compile or at least crashed when running instead of just failing to load anything (the files are different, so something is saved)
         //ar << (const std::vector<const Note*>&)m_vpNotes; qDebug("saved notes sz %d", cSize(m_vpNotes));
 
         std::vector<std::string> v;
@@ -221,16 +225,22 @@ private:
         ar << (const std::vector<std::string>&)v;//*/
     }
 
-    template<class Archive> void load(Archive& ar, const unsigned int /*nVersion*/)
+    template<class Archive> void load(Archive& ar, const unsigned int nVersion)
     {
+        if (nVersion > 0) { throw std::runtime_error("invalid version of serialized file"); }
+
         ar >> m_bNoteFilter;
         ar >> m_bDirFilter;
 
         ar >> m_bSavedNoteFilter;
         ar >> m_bSavedDirFilter;
 
-        ar >> m_vDirs;
-        //qDebug("loaded dirs sz %d", cSize(m_vDirs));
+        ar >> m_vstrDirs;
+        /*qDebug("loaded flt dirs size %d", cSize(m_vstrDirs));
+        for (int i = 0; i < cSize(m_vstrDirs); ++i)
+        {
+            qDebug("flt dir %d: %s", i, m_vstrDirs[i].c_str());
+        }*/
 
         /*std::vector<Note*> v;
         ar >> v;
@@ -403,12 +413,11 @@ public:
     bool useFastSave() const { return m_bFastSave; }
 
 public:
-    enum Case { LOWER, UPPER, TITLE, PHRASE };
-    Case m_eCaseForArtists;
-    Case m_eCaseForOthers;
+    TextCaseOptions m_eCaseForArtists;
+    TextCaseOptions m_eCaseForOthers;
 
     bool m_bWarnOnNonSeqTracks, m_bWarnPastingToNonSeqTracks;
-    bool m_bShowDebug, m_bShowSessions;
+    bool m_bShowExport, m_bShowDebug, m_bShowSessions;
     bool m_bScanAtStartup;
     std::string m_strNormalizeCmd;
     bool m_bKeepNormWndOpen;
@@ -436,6 +445,8 @@ public:
     SessionSettings& m_settings;
 
     bool m_bKeepOneValidImg;
+    bool m_bWmpVarArtists;
+    bool m_bItunesVarArtists;
 
     std::string m_strTransfLog; // log file with transformations;
     bool m_bLogTransf;
@@ -448,10 +459,19 @@ public:
 
     bool m_bWarnedAboutSel;
     bool m_bWarnedAboutBackup;
+    bool m_bToldAboutPatterns; // see also s_bToldAboutPatternsInCrtRun
+    bool m_bToldAboutSupport; // see also s_bToldAboutSupportInCrtRun
+
+    std::string m_strCheckForNewVersions; // really an enum with 3 values
+    QDateTime m_timeLastNewVerCheck;
+    std::string m_strDontTellAboutVer;
 
     bool isTraceToFileEnabled() const { return m_bTraceToFile; }
     void setTraceToFile(bool bTraceToFile); // also removes the file
 
+    std::string m_strRenamerInvalidChars;
+    std::string m_strRenamerReplacementString;
+
 private:
     std::deque<const Mp3Handler*> m_vpAllHandlers; // owns the pointers; sorted by CmpMp3HandlerPtrByName;
 
@@ -501,7 +521,7 @@ private:
 
     void updateUniqueNotes(); // updates m_uniqueNotes to reflect the current m_vpAllHandlers and m_vpFltHandlers;
 
-    mutable int m_nSongInCrtAlbum; // something in the "current album" used by the tag editor; might be first, last or in the middle;
+    mutable int m_nSongInCrtAlbum; // index into m_vpAllHandlers; something in the "current album" used by the tag editor; might be first, last or in the middle;
 
     std::string m_strGenFontName;
     int m_nGenFontSize;
@@ -543,12 +563,17 @@ public slots:
 private:
     friend class boost::serialization::access;
 
-    template<class Archive> void save(Archive& ar, const unsigned int /*nVersion*/) const;
-    template<class Archive> void load(Archive& ar, const unsigned int /*nVersion*/);
+    template<class Archive> void save(Archive& ar, const unsigned int nVersion) const;
+    template<class Archive> void load(Archive& ar, const unsigned int nVersion);
     BOOST_SERIALIZATION_SPLIT_MEMBER();
 };
 
 
+CommonData* getCommonData(); // the only CommonData that exists at a given moment
+
+QWidget* getMainForm();
+
+
 //=====================================================================================================================
 //=====================================================================================================================
 //=====================================================================================================================
@@ -595,7 +620,6 @@ QColor getDefaultBkgCol();
 //=====================================================================================================================
 //=====================================================================================================================
 
-
 void printFontInfo(const char* szLabel, const QFont& font);
 
 // adjusts the global font so it displays legible characters (on Windows it is possible under some unclear circumstances for all characters to be shown as small rectangles)
