@@ -23,18 +23,20 @@
 #ifndef DataStreamH
 #define DataStreamH
 
+#include  <vector>
+
 #include  "SerSupport.h"
 
 #include  "Notes.h"
 #include  "CommonTypes.h"
 
 #define MP3_CHECK(COND, POS, MSG_ID, EXCP) { if (!(COND)) { notes.add(new Note(Notes::MSG_ID(), POS)); throw EXCP; } }
-#define MP3_CHECK_T(COND, POS, MSG, EXCP) { if (!(COND)) { static Note::SharedData d (MSG); notes.add(new Note(d, POS)); throw EXCP; } }
+#define MP3_CHECK_T(COND, POS, MSG, EXCP) { if (!(COND)) { static Note::SharedData d (MSG, false); notes.add(new Note(d, POS)); throw EXCP; } }
 #define MP3_THROW(POS, MSG_ID, EXCP) { notes.add(new Note(Notes::MSG_ID(), POS)); throw EXCP; }
-#define MP3_THROW_T(POS, MSG, EXCP) { static Note::SharedData d (MSG); notes.add(new Note(d, POS)); throw EXCP; }
+#define MP3_THROW_T(POS, MSG, EXCP) { static Note::SharedData d (MSG, false); notes.add(new Note(d, POS)); throw EXCP; }
 #define MP3_NOTE(POS, MSG_ID) { notes.add(new Note(Notes::MSG_ID(), POS)); }
 #define MP3_NOTE_D(POS, MSG_ID, DETAIL) { notes.add(new Note(Notes::MSG_ID(), POS, DETAIL)); }
-#define MP3_TRACE(POS, MSG) { static Note::SharedData d (MSG); notes.add(new Note(d, POS)); }
+#define MP3_TRACE(POS, MSG) { static Note::SharedData d (MSG, false); notes.add(new Note(d, POS)); }
 
 /*
 
@@ -64,7 +66,7 @@ DataStream constructors may leave the input file in EOF or other invalid state a
 
 #define STRM_ASSERT(COND) { if (!(COND)) { assertBreakpoint(); ::trace("assert"); logAssert(__FILE__, __LINE__, #COND, getGlobalMp3HandlerName()); ::exit(1); } }
 
-std::string getGlobalMp3HandlerName(); // a hack to get the name of the current file from inside various streams without storing the name there //ttt1 review
+std::string getGlobalMp3HandlerName(); // a hack to get the name of the current file from inside various streams without storing the name there //ttt2 review
 
 
 class DataStream
@@ -92,8 +94,10 @@ private:
     friend class boost::serialization::access;
 
     template<class Archive>
-    void serialize(Archive& ar, const unsigned int /*nVersion*/)
+    void serialize(Archive& ar, const unsigned int nVersion)
     {
+        if (nVersion > 0) { throw std::runtime_error("invalid version of serialized file"); }
+
         ar & m_nIndex;
     }
 };
@@ -119,8 +123,10 @@ private:
     friend class boost::serialization::access;
 
     template<class Archive>
-    void serialize(Archive& ar, const unsigned int /*nVersion*/)
+    void serialize(Archive& ar, const unsigned int nVersion)
     {
+        if (nVersion > 0) { throw std::runtime_error("invalid version of serialized file"); }
+
         ar & boost::serialization::base_object<DataStream>(*this);
 
         ar & m_pos;
@@ -150,8 +156,10 @@ private:
     friend class boost::serialization::access;
 
     template<class Archive>
-    void serialize(Archive& ar, const unsigned int /*nVersion*/)
+    void serialize(Archive& ar, const unsigned int nVersion)
     {
+        if (nVersion > 0) { throw std::runtime_error("invalid version of serialized file"); }
+
         ar & boost::serialization::base_object<SimpleDataStream>(*this);
 
         ar & m_begin;
@@ -171,8 +179,10 @@ private:
     UnknownDataStream() {} // serialization-only constructor
 
     template<class Archive>
-    void serialize(Archive& ar, const unsigned int /*nVersion*/)
+    void serialize(Archive& ar, const unsigned int nVersion)
     {
+        if (nVersion > 0) { throw std::runtime_error("invalid version of serialized file"); }
+
         ar & boost::serialization::base_object<UnknownDataStreamBase>(*this);
     }
 };
@@ -199,8 +209,10 @@ private:
     BrokenDataStream() {} // serialization-only constructor
 
     template<class Archive>
-    void serialize(Archive& ar, const unsigned int /*nVersion*/)
+    void serialize(Archive& ar, const unsigned int nVersion)
     {
+        if (nVersion > 0) { throw std::runtime_error("invalid version of serialized file"); }
+
         ar & boost::serialization::base_object<UnknownDataStreamBase>(*this);
 
         ar & m_strName;
@@ -216,11 +228,7 @@ class UnsupportedDataStream : public UnknownDataStreamBase // ttt3 perhaps merge
     std::string m_strBaseName;
     std::string m_strInfo;
 public:
-    UnsupportedDataStream(int nIndex, NoteColl& notes, std::istream& in, std::streamoff nSize, const char* szBaseName, const std::string& strInfo) :
-            UnknownDataStreamBase(nIndex, notes, in, nSize),
-            m_strName(std::string("Unsupported ") + szBaseName),
-            m_strBaseName(szBaseName),
-            m_strInfo(strInfo.empty() ? UnknownDataStreamBase::getInfo() : strInfo + "; " + UnknownDataStreamBase::getInfo()) {}
+    UnsupportedDataStream(int nIndex, NoteColl& notes, std::istream& in, std::streamoff nSize, const char* szBaseName, const std::string& strInfo);
 
     /*override*/ const char* getDisplayName() const { return m_strName.c_str(); } // DECL_NAME doesn't work in this case
     /*override*/ std::string getInfo() const { return m_strInfo; }
@@ -231,8 +239,10 @@ private:
     UnsupportedDataStream() {} // serialization-only constructor
 
     template<class Archive>
-    void serialize(Archive& ar, const unsigned int /*nVersion*/)
+    void serialize(Archive& ar, const unsigned int nVersion)
     {
+        if (nVersion > 0) { throw std::runtime_error("invalid version of serialized file"); }
+
         ar & boost::serialization::base_object<UnknownDataStreamBase>(*this);
 
         ar & m_strName;
@@ -243,7 +253,7 @@ private:
 
 
 class MpegStream;
-class MpegFrameBase;
+struct MpegFrameBase;
 
 
 // follows an MPEG stream and is compatible with it but it is truncated; some other frame begins before where it should end
@@ -270,8 +280,10 @@ private:
     TruncatedMpegDataStream() {} // serialization-only constructor
 
     template<class Archive>
-    void serialize(Archive& ar, const unsigned int /*nVersion*/)
+    void serialize(Archive& ar, const unsigned int nVersion)
     {
+        if (nVersion > 0) { throw std::runtime_error("invalid version of serialized file"); }
+
         ar & boost::serialization::base_object<UnknownDataStreamBase>(*this);
 
         ar & m_pFrame;
@@ -302,8 +314,10 @@ private:
     NullDataStream() {} // serialization-only constructor
 
     template<class Archive>
-    void serialize(Archive& ar, const unsigned int /*nVersion*/)
+    void serialize(Archive& ar, const unsigned int nVersion)
     {
+        if (nVersion > 0) { throw std::runtime_error("invalid version of serialized file"); }
+
         ar & boost::serialization::base_object<DataStream>(*this);
 
         ar & m_pos;
@@ -411,7 +425,7 @@ struct TagReader
 {
     virtual ~TagReader();
 
-    enum Feature { TITLE, ARTIST, TRACK_NUMBER, TIME, GENRE, IMAGE, ALBUM, RATING, COMPOSER, LIST_END };
+    enum Feature { TITLE, ARTIST, TRACK_NUMBER, TIME, GENRE, IMAGE, ALBUM, RATING, COMPOSER, VARIOUS_ARTISTS, LIST_END };
     static const char* getLabel(int); // text representation for each Feature
 
     enum SuportLevel { NOT_SUPPORTED, READ_ONLY/*, READ_WRITE*/ };
@@ -436,14 +450,24 @@ struct TagReader
 
     virtual std::string getComposer(bool* /*pbFrameExists*/ = 0) const { throw NotSupportedOp(); } // UTF8
 
+    enum VariousArtists { VA_NONE = 0, VA_ITUNES = 1, VA_WMP = 2 };
+
+    virtual int getVariousArtists(bool* /*pbFrameExists*/ = 0) const { throw NotSupportedOp(); } // combination of VariousArtists flags; since several frames might be involved, *pbFrameExists is set to "true" if at least a frame exists
+
     virtual std::string getOtherInfo() const { return ""; } // non-editable tags
 
+    virtual std::vector<ImageInfo> getImages() const { return std::vector<ImageInfo>(); } // all images, even those with errors
+
+    virtual std::string getImageData(bool* pbFrameExists = 0) const { if (0 != pbFrameExists) { *pbFrameExists = false; } return ""; }
+
     virtual SuportLevel getSupport(Feature) const { return NOT_SUPPORTED; }
 
     virtual const char* getName() const = 0;
 
     std::string getValue(Feature) const; // returns the corresponding feature converted to a string; if it's not supported, it returns an empty string; for IMAGE an empty string regardless of a picture being present or not
 
+    static std::string getVarArtistsValue(); // what getValue(VARIOUS_ARTISTS) returns for VA tags, based on global configuration
+
     static int FEATURE_ON_POS[]; // the order in which Features should appear in grids
     static int POS_OF_FEATURE[]; // the "inverse" of FEATURE_ON_POS: what Feature appears in a given position
 };
