@@ -25,7 +25,6 @@
 #include  <QTimer>
 #include  <QPainter>
 #include  <QHeaderView>
-#include  <QSizeGrip>
 
 #include  "FileRenamerDlgImpl.h"
 
@@ -65,6 +64,15 @@ void HndlrListModel::setRenamer(const Renamer* p)
 }
 
 
+void HndlrListModel::setUnratedAsDuplicates(bool bUnratedAsDuplicate)
+{
+    if (0 == m_pRenamer) { return; }
+
+    m_pRenamer->m_bUnratedAsDuplicate = bUnratedAsDuplicate;
+    emitLayoutChanged();
+}
+
+
 // returns either m_pCommonData->getCrtAlbum() or m_pCommonData->getViewHandlers(), based on m_bUseCurrentView
 const deque<const Mp3Handler*> HndlrListModel::getHandlerList() const
 {
@@ -84,6 +92,16 @@ const deque<const Mp3Handler*> HndlrListModel::getHandlerList() const
 }
 
 
+/*override*/ Qt::ItemFlags HndlrListModel::flags(const QModelIndex& index) const
+{
+    Qt::ItemFlags flg (QAbstractTableModel::flags(index));
+    if (1 == index.column() && 0 != m_pRenamer)
+    {
+        flg = flg | Qt::ItemIsEditable;
+    }
+    return flg;
+}
+
 /*override*/ QVariant HndlrListModel::data(const QModelIndex& index, int nRole) const
 {
 LAST_STEP("HndlrListModel::data()");
@@ -91,7 +109,8 @@ LAST_STEP("HndlrListModel::data()");
     int i (index.row());
     int j (index.column());
 
-    if (Qt::DisplayRole != nRole && Qt::ToolTipRole != nRole) { return QVariant(); }
+    if (Qt::DisplayRole != nRole && Qt::ToolTipRole != nRole && Qt::EditRole != nRole) { return QVariant(); }
+
     QString s;
 
     const Mp3Handler* p (getHandlerList().at(i));
@@ -134,7 +153,7 @@ LAST_STEP("HndlrListModel::data()");
         s = convStr(pId3V2->getValue((TagReader::Feature)TagReader::FEATURE_ON_POS[j]));
     }
 
-    if (Qt::DisplayRole == nRole)
+    if (Qt::DisplayRole == nRole || Qt::EditRole == nRole)
     {
         return s;
     }
@@ -155,6 +174,17 @@ LAST_STEP("HndlrListModel::data()");
 }
 
 
+/*override*/ bool HndlrListModel::setData(const QModelIndex& index, const QVariant& value, int nRole /*= Qt::EditRole*/)
+{
+    if (Qt::EditRole != nRole) { return false; }
+
+    CB_ASSERT (0 != m_pRenamer);
+
+    m_pRenamer->m_mValues[getHandlerList().at(index.row())] = convStr(fromNativeSeparators(value.toString()));
+    return true;
+}
+
+
 
 /*override*/ QVariant HndlrListModel::headerData(int nSection, Qt::Orientation eOrientation, int nRole /*= Qt::DisplayRole*/) const
 {
@@ -198,10 +228,26 @@ CurrentAlbumDelegate::CurrentAlbumDelegate(QWidget* pParent, HndlrListModel* pHn
 
     if (0 == pId3V2)
     {
-        //pPainter->fillRect(option.rect, QColor(255, 226, 236)); //ttt1 perhaps put back, but should work for "missing fields" as well
+        //pPainter->fillRect(option.rect, QColor(255, 226, 236)); //ttt2 perhaps put back, but should work for "missing fields" as well
     }
 
-    QItemDelegate::paint(pPainter, option, index);
+    QStyleOptionViewItemV2 myOption (option);
+
+    if (0 != m_pHndlrListModel->getRenamer() && index.column() == 1)
+    {
+        string strNewName (m_pHndlrListModel->getRenamer()->getNewName(p));
+        if (fileExists(strNewName))
+        {
+            pPainter->fillRect(option.rect, strNewName == p->getName() ? QColor(226, 236, 255) : QColor(255, 226, 236));
+        }
+
+        if (m_pHndlrListModel->getRenamer()->m_mValues.count(p) > 0)
+        {
+            myOption.font.setItalic(true);
+        }
+    }
+
+    QItemDelegate::paint(pPainter, myOption, index);
 
     pPainter->restore();
 }
@@ -214,7 +260,7 @@ CurrentAlbumDelegate::CurrentAlbumDelegate(QWidget* pParent, HndlrListModel* pHn
 
 
 
-FileRenamerDlgImpl::FileRenamerDlgImpl(QWidget* pParent, CommonData* pCommonData, bool bUseCurrentView) : QDialog(pParent, getDialogWndFlags()), Ui::FileRenamerDlg(), m_pCommonData(pCommonData), m_bUseCurrentView(bUseCurrentView)
+FileRenamerDlgImpl::FileRenamerDlgImpl(QWidget* pParent, CommonData* pCommonData, bool bUseCurrentView) : QDialog(pParent, getDialogWndFlags()), Ui::FileRenamerDlg(), m_pCommonData(pCommonData), m_bUseCurrentView(bUseCurrentView), m_pEditor(0)
 {
     setupUi(this);
 
@@ -231,6 +277,8 @@ FileRenamerDlgImpl::FileRenamerDlgImpl(QWidget* pParent, CommonData* pCommonData
 
         CurrentAlbumDelegate* pDel (new CurrentAlbumDelegate(this, m_pHndlrListModel));
         m_pCurrentAlbumG->setItemDelegate(pDel);
+
+        m_pCurrentAlbumG->viewport()->installEventFilter(this);
     }
 
     m_pButtonGroup = new QButtonGroup(this);
@@ -240,8 +288,8 @@ FileRenamerDlgImpl::FileRenamerDlgImpl(QWidget* pParent, CommonData* pCommonData
 
     {
         int nWidth, nHeight;
-        bool bKeepOriginal;
-        m_pCommonData->m_settings.loadRenamerSettings(nWidth, nHeight, m_nSaButton, m_nVaButton, bKeepOriginal);
+        bool bKeepOriginal, bUnratedAsDuplicate;
+        m_pCommonData->m_settings.loadRenamerSettings(nWidth, nHeight, m_nSaButton, m_nVaButton, bKeepOriginal, bUnratedAsDuplicate);
         if (nWidth > 400 && nHeight > 400)
         {
             resize(nWidth, nHeight);
@@ -254,6 +302,7 @@ FileRenamerDlgImpl::FileRenamerDlgImpl(QWidget* pParent, CommonData* pCommonData
         if (m_nVaButton >= cSize(m_vstrPatterns) || m_nVaButton <= 0) { m_nVaButton = 0; }
         if (m_nSaButton >= cSize(m_vstrPatterns) || m_nSaButton <= 0) { m_nSaButton = 0; }
         m_pKeepOriginalCkB->setChecked(bKeepOriginal);
+        m_pMarkUnratedAsDuplicatesCkB->setChecked(bUnratedAsDuplicate);
     }
 
 
@@ -274,7 +323,7 @@ FileRenamerDlgImpl::FileRenamerDlgImpl(QWidget* pParent, CommonData* pCommonData
 
 FileRenamerDlgImpl::~FileRenamerDlgImpl()
 {
-    m_pCommonData->m_settings.saveRenamerSettings(width(), height(), m_nSaButton, m_nVaButton, m_pKeepOriginalCkB->isChecked());
+    m_pCommonData->m_settings.saveRenamerSettings(width(), height(), m_nSaButton, m_nVaButton, m_pKeepOriginalCkB->isChecked(), m_pMarkUnratedAsDuplicatesCkB->isChecked());
 }
 
 
@@ -291,6 +340,7 @@ string FileRenamerDlgImpl::run()
 
 void FileRenamerDlgImpl::on_m_pNextB_clicked()
 {
+    closeEditor();
     if (m_pCommonData->nextAlbum())
     {
         reloadTable();
@@ -299,6 +349,7 @@ void FileRenamerDlgImpl::on_m_pNextB_clicked()
 
 void FileRenamerDlgImpl::on_m_pPrevB_clicked()
 {
+    closeEditor();
     if (m_pCommonData->prevAlbum())
     {
         reloadTable();
@@ -322,30 +373,28 @@ void FileRenamerDlgImpl::on_m_pEditPatternsB_clicked()
 void FileRenamerDlgImpl::updateButtons()
 {
     m_nBtnId = 0;
-    //createButtons(m_pSaButtonsW);
-    //createButtons(m_pVaButtonsW);
-    createButtons(m_pButtonsW);
+    createButtons();
 }
 
 
-void FileRenamerDlgImpl::createButtons(QWidget* pWidget)
+void FileRenamerDlgImpl::createButtons()
 {
-    QBoxLayout* pLayout (dynamic_cast<QBoxLayout*>(pWidget->layout()));
+    QBoxLayout* pLayout (dynamic_cast<QBoxLayout*>(m_pButtonsW->layout()));
     CB_ASSERT (0 != pLayout);
     /*int nPos (pLayout->indexOf(pOldBtn));
     pLayout->insertWidget(nPos, this);*/
 
-    QObjectList l (pWidget->children());
+    QObjectList l (m_pButtonsW->children());
     //qDebug("cnt: %d", l.size());
 
-    for (int i = 1, n = l.size(); i < n; ++i)
+    for (int i = 1, n = l.size(); i < n; ++i) // l[0] is m_pButtonsW's layout (note that m_pAlbumTypeL is in m_pBtnPanelW)
     {
         delete l[i];
     }
 
     for (int i = 0, n = cSize(m_vstrPatterns); i < n; ++i)
     {
-        QToolButton* p (new QToolButton(pWidget));
+        QToolButton* p (new QToolButton(m_pButtonsW));
         p->setText(toNativeSeparators(convStr(m_vstrPatterns[i])));
         p->setCheckable(true);
         m_pButtonGroup->addButton(p, m_nBtnId++);
@@ -373,7 +422,44 @@ void FileRenamerDlgImpl::onPatternClicked()
     {
         m_nVaButton = nId;
     }
-    m_pHndlrListModel->setRenamer(new Renamer(m_vstrPatterns[nId]));
+    m_pHndlrListModel->setRenamer(new Renamer(m_vstrPatterns[nId], m_pCommonData, m_pMarkUnratedAsDuplicatesCkB->isChecked()));
+    resizeUi();
+}
+
+
+void FileRenamerDlgImpl::closeEditor()
+{
+    if (0 != m_pEditor)
+    {
+        delete m_pEditor;
+    }
+}
+
+
+/*override*/ bool FileRenamerDlgImpl::eventFilter(QObject* pObj, QEvent* pEvent)
+{
+//qDebug("ev %d", int(pEvent->type()));
+    if (QEvent::ChildAdded == pEvent->type())
+    {
+        //qDebug("add");
+        QObject* pChild (((QChildEvent*)pEvent)->child());
+        if (pChild->isWidgetType())
+        {
+            CB_ASSERT (0 == m_pEditor);
+            m_pEditor = pChild;
+        }
+    }
+    else if (QEvent::ChildRemoved == pEvent->type())
+    {
+        //qDebug("rm");
+        QObject* pChild (((QChildEvent*)pEvent)->child());
+        if (pChild->isWidgetType())
+        {
+            CB_ASSERT (pChild == m_pEditor);
+            m_pEditor = 0;
+        }
+    }
+    return QDialog::eventFilter(pObj, pEvent);
 }
 
 
@@ -611,7 +697,7 @@ void FileRenamerDlgImpl::on_m_pRenameB_clicked()
 
     m_pCommonData->mergeHandlerChanges(vpAdd, vpDel, CommonData::SEL | CommonData::CURRENT);
 
-    if (!bKeepOrig)
+    if (!bKeepOrig || pRenamer->isSameDir())
     {
         reloadTable();
     }
@@ -703,12 +789,12 @@ void FileRenamerDlgImpl::selectPattern()
         if (isSingleArtist())
         {
             m_pButtonGroup->button(m_nSaButton)->setChecked(true);
-            m_pHndlrListModel->setRenamer(new Renamer(m_vstrPatterns[m_nSaButton]));
+            m_pHndlrListModel->setRenamer(new Renamer(m_vstrPatterns[m_nSaButton], m_pCommonData, m_pMarkUnratedAsDuplicatesCkB->isChecked()));
         }
         else
         {
             m_pButtonGroup->button(m_nVaButton /*+ cSize(m_vstrPatterns)*/)->setChecked(true);
-            m_pHndlrListModel->setRenamer(new Renamer(m_vstrPatterns[m_nVaButton]));
+            m_pHndlrListModel->setRenamer(new Renamer(m_vstrPatterns[m_nVaButton], m_pCommonData, m_pMarkUnratedAsDuplicatesCkB->isChecked()));
         }
     }
     else
@@ -718,7 +804,7 @@ void FileRenamerDlgImpl::selectPattern()
 
     resizeUi();
 
-    m_pAlbumTypeL->setText(isSingleArtist() ? "Single artist" : "Various artists"); //ttt1 see if "single" is the best word
+    m_pAlbumTypeL->setText(isSingleArtist() ? "Single artist" : "Various artists"); //ttt2 see if "single" is the best word
 }
 
 
@@ -736,7 +822,7 @@ void FileRenamerDlgImpl::loadPatterns()
         string strPatt (v[i]);
         try
         {
-            Renamer r (strPatt);
+            Renamer r (strPatt, m_pCommonData, m_pMarkUnratedAsDuplicatesCkB->isChecked());
             m_vstrPatterns.push_back(strPatt);
         }
         catch (const Renamer::InvalidPattern&)
@@ -745,9 +831,9 @@ void FileRenamerDlgImpl::loadPatterns()
         }
     }
 
-    /*if (m_vstrPatterns.empty()) // ttt1 because there is no default, the user is forced to add a pattern first; see if a meaningful default can be used
+    /*if (m_vstrPatterns.empty()) // ttt2 because there is no default, the user is forced to add a pattern first; see if a meaningful default can be used
     { // use default (only if the user didn't remove all patterns on purpose)
-        string s ("/tmp/%a/%b[ %y]/%r%n %t"); //ttt1 change from tmp to root/out; //ttt1 perhaps ask the user //ttt1 OS specific
+        string s ("/tmp/%a/%b[ %y]/%r%n %t"); //ttt2 change from tmp to root/out; //ttt2 perhaps ask the user //ttt2 OS specific
         Renamer r (s); // may throw after porting, but that's OK, because it signals a fix is needed
         m_vstrPatterns.push_back(s);
     }*/
@@ -766,7 +852,7 @@ void FileRenamerDlgImpl::savePatterns()
 }
 
 
-//ttt1 perhaps have a "reload" button
+//ttt2 perhaps have a "reload" button
 
 void FileRenamerDlgImpl::resizeIcons()
 {
@@ -794,6 +880,13 @@ void FileRenamerDlgImpl::onHelp()
 }
 
 
+//void FileRenamerDlgImpl::on_m_pMarkUnratedAsDuplicatesCkB_stateChanged()
+void FileRenamerDlgImpl::on_m_pMarkUnratedAsDuplicatesCkB_clicked()
+{
+    m_pHndlrListModel->setUnratedAsDuplicates(m_pMarkUnratedAsDuplicatesCkB->isChecked());
+}
+
+
 
 //======================================================================================================================
 //======================================================================================================================
@@ -801,15 +894,45 @@ void FileRenamerDlgImpl::onHelp()
 
 namespace FileRenamer {
 
+
+class InvalidCharsReplacer
+{
+    string m_strRenamerInvalidChars;
+    string m_strRenamerReplacementString;
+public:
+    std::string fixName(std::string s) const;
+    InvalidCharsReplacer(const string& strRenamerInvalidChars, const string& strRenamerReplacementString) : m_strRenamerInvalidChars(strRenamerInvalidChars), m_strRenamerReplacementString(strRenamerReplacementString) {}
+};
+
+
+string InvalidCharsReplacer::fixName(string s) const
+{
+    CB_ASSERT (string::npos == m_strRenamerReplacementString.find_first_of(m_strRenamerInvalidChars));
+    if (m_strRenamerInvalidChars.empty()) { return s; }
+
+    for (;;)
+    {
+        string::size_type n (s.find_first_of(m_strRenamerInvalidChars));
+        if (string::npos == n) { break; }
+        s.replace(n, 1, m_strRenamerReplacementString);
+    }
+    return s;
+}
+
+
+
+
 struct PatternBase
 {
+    const InvalidCharsReplacer* m_pInvalidCharsReplacer;
+    PatternBase(const InvalidCharsReplacer* pInvalidCharsReplacer) : m_pInvalidCharsReplacer(pInvalidCharsReplacer) {}
     virtual ~PatternBase() {}
     virtual string getVal(const Mp3Handler*) const = 0;
 };
 
 struct StaticPattern : public PatternBase
 {
-    StaticPattern(string strVal) : m_strVal(strVal) {}
+    StaticPattern(string strVal, const InvalidCharsReplacer* pInvalidCharsReplacer) : PatternBase(pInvalidCharsReplacer), m_strVal(strVal) {}
     /*override*/ string getVal(const Mp3Handler*) const { return m_strVal; }
 private:
     string m_strVal;
@@ -818,32 +941,20 @@ private:
 
 struct FieldPattern : public PatternBase
 {
-    FieldPattern(TagReader::Feature eFeature) : m_eFeature(eFeature) {}
+    FieldPattern(TagReader::Feature eFeature, const InvalidCharsReplacer* pInvalidCharsReplacer) : PatternBase(pInvalidCharsReplacer), m_eFeature(eFeature) {}
     /*override*/ string getVal(const Mp3Handler* pHndl) const
     {
         const Id3V2StreamBase* p (pHndl->getId3V2Stream());
         if (0 == p) { return ""; }
-        return fixName(p->getValue(m_eFeature));
+        return m_pInvalidCharsReplacer->fixName(p->getValue(m_eFeature));
     }
 private:
     TagReader::Feature m_eFeature;
-    string fixName(string s) const
-    {
-        for (;;)
-        {
-#ifndef WIN32
-            string::size_type n (s.find_first_of("/\"\\*?<>|")); //ttt2 even in Unix, it might be a good idea to not allow ":" as well in some cases, depending on the file system
-#else
-            string::size_type n (s.find_first_of("/\"\\*?<>|:"));
-#endif
-            if (string::npos == n) { return s; }
-            s[n] = '_';
-        }
-    }
 };
 
 struct YearPattern : public PatternBase
 {
+    YearPattern(const InvalidCharsReplacer* pInvalidCharsReplacer) : PatternBase(pInvalidCharsReplacer) {}
     /*override*/ string getVal(const Mp3Handler* pHndl) const
     {
         const Id3V2StreamBase* p (pHndl->getId3V2Stream());
@@ -854,13 +965,14 @@ struct YearPattern : public PatternBase
 
 struct TrackNoPattern : public PatternBase
 {
+    TrackNoPattern(const InvalidCharsReplacer* pInvalidCharsReplacer) : PatternBase(pInvalidCharsReplacer) {}
     /*override*/ string getVal(const Mp3Handler* pHndl) const
     {
         const Id3V2StreamBase* p (pHndl->getId3V2Stream());
-        if (0 == p) { return ""; }
+        if (0 == p) { return "00"; }
         string s (p->getTrackNumber());
         int n (atoi(s.c_str()));
-        if (n <= 0) { return ""; }
+        if (n <= 0) { return "00"; }
         char a [20];
         sprintf(a, "%02d", n);
         return a;
@@ -871,12 +983,14 @@ struct TrackNoPattern : public PatternBase
 
 struct RatingPattern : public PatternBase
 {
+    RatingPattern(const InvalidCharsReplacer* pInvalidCharsReplacer, const bool& bUnratedAsDuplicate) : PatternBase(pInvalidCharsReplacer), m_bUnratedAsDuplicate(bUnratedAsDuplicate) {}
+    const bool& m_bUnratedAsDuplicate;
     /*override*/ string getVal(const Mp3Handler* pHndl) const
     {
         const Id3V2StreamBase* p (pHndl->getId3V2Stream());
         if (0 == p) { return ""; }
         double r (p->getRating());
-        if (r < 0) { return ""; }
+        if (r < 0) { return m_bUnratedAsDuplicate ? "q" : ""; }
 // from TrackTextReader::TrackTextReader
 //                                   a    b    c    d    e    f    g    h    i    j    k    l    m    n    o    p   q   r    s    t    u    v    w    x    y    z
 //static double s_ratingMap [] = { 5.0, 4.5, 4.0, 3.5, 3.0, 2.5, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 1.5, 1.0, 1.0, 1.0, -1, -1, 1.0, 1.0, 1.0, 1.0, 1.0, 0.5, 0.5, 0.0 };
@@ -897,6 +1011,7 @@ struct RatingPattern : public PatternBase
 
 struct SequencePattern : public PatternBase
 {
+    SequencePattern(const InvalidCharsReplacer* pInvalidCharsReplacer) : PatternBase(pInvalidCharsReplacer) {}
     /*override*/ ~SequencePattern() { clearPtrContainer(m_vpPatterns); }
     /*override*/ string getVal(const Mp3Handler* pHndl) const { return getVal(pHndl, ACCEPT_EMPTY); }
     string getNonNullVal(const Mp3Handler* pHndl) const { return getVal(pHndl, DONT_ACCEPT_EMPTY); } // returns an empty string if any of its components are empty
@@ -920,7 +1035,7 @@ private:
 struct OptionalPattern : public PatternBase
 {
     /*override*/ ~OptionalPattern() { delete m_pSequencePattern; }
-    OptionalPattern(SequencePattern* pSequencePattern) : m_pSequencePattern(pSequencePattern) {}
+    OptionalPattern(SequencePattern* pSequencePattern, const InvalidCharsReplacer* pInvalidCharsReplacer) : PatternBase(pInvalidCharsReplacer), m_pSequencePattern(pSequencePattern) {}
     /*override*/ string getVal(const Mp3Handler* pHndl) const { return m_pSequencePattern->getNonNullVal(pHndl); }
 private:
     SequencePattern* m_pSequencePattern;
@@ -931,8 +1046,19 @@ private:
 //using namespace RenamerPatterns;
 
 
-Renamer::Renamer(const std::string& strPattern) : m_strPattern(strPattern), m_pRoot(new SequencePattern()), m_bSameDir(string::npos == strPattern.find(getPathSep()))
+Renamer::Renamer(const std::string& strPattern, const CommonData* pCommonData, bool bUnratedAsDuplicate1) : m_strPattern(strPattern), m_bSameDir(string::npos == strPattern.find(getPathSep())), m_pCommonData(pCommonData), m_bUnratedAsDuplicate(bUnratedAsDuplicate1)
 {
+    if (0 != pCommonData)
+    {
+        m_pInvalidCharsReplacer.reset(new InvalidCharsReplacer(pCommonData->m_strRenamerInvalidChars, pCommonData->m_strRenamerReplacementString));
+    }
+    else
+    {
+        m_pInvalidCharsReplacer.reset(new InvalidCharsReplacer("", ""));
+    }
+
+    m_pRoot = new SequencePattern(m_pInvalidCharsReplacer.get());
+
     auto_ptr<SequencePattern> ap (m_pRoot);
     const char* p (strPattern.c_str());
 
@@ -953,7 +1079,7 @@ Renamer::Renamer(const std::string& strPattern) : m_strPattern(strPattern), m_pR
             throw InvalidPattern(strPattern, "A pattern must either begin with \"<drive>:\\\" or contain no '\\' at all");
         }
         p += 2;
-        m_pRoot->addPattern(new StaticPattern(strPattern.substr(0, 2)));
+        m_pRoot->addPattern(new StaticPattern(strPattern.substr(0, 2), m_pInvalidCharsReplacer.get()));
         if (getPathSep() != *p) { throw InvalidPattern(strPattern, "A pattern must either begin with \"<drive>:\\\" or contain no '\\' at all"); }
 
 #endif
@@ -989,19 +1115,20 @@ Renamer::Renamer(const std::string& strPattern) : m_strPattern(strPattern), m_pR
                 case 'g':
                 case 'c':
                 case 'r':
-                    if (!strStatic.empty()) { pSeq->addPattern(new StaticPattern(strStatic)); strStatic.clear(); }
+                    if (!strStatic.empty()) { pSeq->addPattern(new StaticPattern(strStatic, m_pInvalidCharsReplacer.get())); strStatic.clear(); }
                 }
 
                 switch (c1)
                 {
-                case 'n': pSeq->addPattern(new TrackNoPattern()); break;
-                case 'a': pSeq->addPattern(new FieldPattern(TagReader::ARTIST)); break;
-                case 't': pSeq->addPattern(new FieldPattern(TagReader::TITLE)); bTitleFound = true; break;
-                case 'b': pSeq->addPattern(new FieldPattern(TagReader::ALBUM)); break;
-                case 'y': pSeq->addPattern(new YearPattern()); break;
-                case 'g': pSeq->addPattern(new FieldPattern(TagReader::GENRE)); break;
-                case 'c': pSeq->addPattern(new FieldPattern(TagReader::COMPOSER)); break;
-                case 'r': pSeq->addPattern(new RatingPattern()); break;
+                case 'n': pSeq->addPattern(new TrackNoPattern(m_pInvalidCharsReplacer.get())); break;
+                case 'a': pSeq->addPattern(new FieldPattern(TagReader::ARTIST, m_pInvalidCharsReplacer.get())); break;
+                case 't': pSeq->addPattern(new FieldPattern(TagReader::TITLE, m_pInvalidCharsReplacer.get())); bTitleFound = true; break;
+                case 'b': pSeq->addPattern(new FieldPattern(TagReader::ALBUM, m_pInvalidCharsReplacer.get())); break;
+                case 'y': pSeq->addPattern(new YearPattern(m_pInvalidCharsReplacer.get())); break;
+                case 'g': pSeq->addPattern(new FieldPattern(TagReader::GENRE, m_pInvalidCharsReplacer.get())); break;
+                case 'c': pSeq->addPattern(new FieldPattern(TagReader::COMPOSER, m_pInvalidCharsReplacer.get())); break;
+                //ttt2 perhaps add something for "various artists"
+                case 'r': pSeq->addPattern(new RatingPattern(m_pInvalidCharsReplacer.get(), m_bUnratedAsDuplicate)); break;
 
                 case '%':
                 case '[':
@@ -1013,7 +1140,7 @@ Renamer::Renamer(const std::string& strPattern) : m_strPattern(strPattern), m_pR
                     {
                         ostringstream s;
                         s << "Error in column " << p - strPattern.c_str() <<  ".";
-                        throw InvalidPattern(strPattern, s.str()); // ttt1 more details, perhaps make tag edt errors more similar to this
+                        throw InvalidPattern(strPattern, s.str()); // ttt2 more details, perhaps make tag edt errors more similar to this
                     }
                 }
             }
@@ -1021,18 +1148,18 @@ Renamer::Renamer(const std::string& strPattern) : m_strPattern(strPattern), m_pR
 
         case '[':
             {
-                if (!strStatic.empty()) { pSeq->addPattern(new StaticPattern(strStatic)); strStatic.clear(); }
+                if (!strStatic.empty()) { pSeq->addPattern(new StaticPattern(strStatic, m_pInvalidCharsReplacer.get())); strStatic.clear(); }
                 if (pSeq != m_pRoot) { throw InvalidPattern(strPattern, "Nested optional elements are not allowed"); } //ttt2 column
-                pSeq = new SequencePattern();
+                pSeq = new SequencePattern(m_pInvalidCharsReplacer.get());
                 optAp.reset(pSeq);
                 break;
             }
 
         case ']':
             {
-                if (!strStatic.empty()) { pSeq->addPattern(new StaticPattern(strStatic)); strStatic.clear(); }
+                if (!strStatic.empty()) { pSeq->addPattern(new StaticPattern(strStatic, m_pInvalidCharsReplacer.get())); strStatic.clear(); }
                 if (pSeq == m_pRoot) { throw InvalidPattern(strPattern, "Trying to close and optional element although none is open"); } //ttt2 column
-                m_pRoot->addPattern(new OptionalPattern(pSeq));
+                m_pRoot->addPattern(new OptionalPattern(pSeq, m_pInvalidCharsReplacer.get()));
                 pSeq = m_pRoot;
                 optAp.release();
                 break;
@@ -1043,7 +1170,7 @@ Renamer::Renamer(const std::string& strPattern) : m_strPattern(strPattern), m_pR
         }
     }
 
-    if (!strStatic.empty()) { pSeq->addPattern(new StaticPattern(strStatic)); strStatic.clear(); }
+    if (!strStatic.empty()) { pSeq->addPattern(new StaticPattern(strStatic, m_pInvalidCharsReplacer.get())); strStatic.clear(); }
 
     if (pSeq != m_pRoot) { throw InvalidPattern(strPattern, "Optional element must be closed"); } //ttt2 column
 
@@ -1061,6 +1188,8 @@ Renamer::~Renamer()
 string Renamer::getNewName(const Mp3Handler* pHndl) const
 {
     if (0 == pHndl->getId3V2Stream()) { return ""; }
+    if (m_mValues.count(pHndl) > 0) { return m_mValues[pHndl]; }
+
     string s (m_pRoot->getVal(pHndl));
     CB_ASSERT (!m_bSameDir ^ (string::npos == s.find(getPathSep())));
     if (m_bSameDir)
@@ -1073,7 +1202,8 @@ string Renamer::getNewName(const Mp3Handler* pHndl) const
     return s;
 }
 
-//ttt1 timer in normalizer
-//ttt1 look at normalized loudness in tracks, maybe warn
 
+//ttt2 should be possible to filter by var/single artists and do the renaming for all; or have a checkbox in the renamer, but that requires the renamer to have the concept of an album
+//ttt2 add "reload()"
+//ttt2 add palette
 
