@@ -284,7 +284,7 @@ void FilesModel::matchSelToNotes()
 
     m_pCommonData->m_pFilesG->selectionModel()->clearSelection();
 
-    for (set<int, int>::iterator it = sSel.begin(), end = sSel.end(); it != end; ++it)
+    for (set<int>::const_iterator it = sSel.begin(), end = sSel.end(); it != end; ++it)
     {
         m_pCommonData->m_pFilesG->selectionModel()->select(index(m_pCommonData->getFilesGCrtRow(), *it + 1), QItemSelectionModel::Select);
 //cout << "selecting " << m_pCommonData->getFilesGCrtRow() << "x" << *it + 1 << endl;
@@ -470,7 +470,7 @@ case 3:
 
 
 
-FileHeaderView::FileHeaderView(CommonData* pCommonData, Qt::Orientation orientation, QWidget* pParent) : QHeaderView(orientation, pParent), m_pCommonData(pCommonData)
+FileHeaderView::FileHeaderView(CommonData* pCommonData, QWidget* pParent) : QHeaderView(Qt::Horizontal, pParent), m_pCommonData(pCommonData)
 {
 }
 
@@ -486,6 +486,10 @@ FileHeaderView::FileHeaderView(CommonData* pCommonData, Qt::Orientation orientat
 }
 
 
+static int s_nCut (0);
+
+int getHeaderDrawOffset() { return s_nCut; }
+
 /*override*/ void FileHeaderView::paintSection(QPainter* pPainter, const QRect& r, int nLogicalIndex) const
 {
     if (0 == nLogicalIndex)
@@ -501,7 +505,7 @@ FileHeaderView::FileHeaderView(CommonData* pCommonData, Qt::Orientation orientat
 
     pPainter->save();
 
-    // partial copy from Qt's implementation of QHeaderView
+    // partial copy from Qt's implementation of QHeaderView (qheaderview.cpp)
     QStyleOptionHeader opt;
     initStyleOption(&opt);
 
@@ -522,7 +526,6 @@ FileHeaderView::FileHeaderView(CommonData* pCommonData, Qt::Orientation orientat
 
     style()->drawControl(QStyle::CE_Header, &opt, pPainter, this);
 
-    static int s_nCut (0);
     static bool s_bCutInit (false);
     if (!s_bCutInit)
     {
