@@ -42,17 +42,45 @@ using namespace std;
 //ttt2 remove id3v1; copy id3v2 to id3v1; copy id3v1 to id3v2;
 
 
-// ttt1 another option: before a group of transforms is executed on a group of files, a function is called on transforms to allow them to ask config info. most won't use this ...
+// ttt2 another option: before a group of transforms is executed on a group of files, a function is called on transforms to allow them to ask config info. most won't use this ...
 
 
 
 bool Id3V2Cleaner::processId3V2Stream(Id3V2StreamBase& strm, ofstream_utf8& out)
 {
-    Id3V230StreamWriter wrt (m_pCommonData->m_bKeepOneValidImg, m_pCommonData->useFastSave(), 0);
+    Id3V230StreamWriter wrt (m_pCommonData->m_bKeepOneValidImg, m_pCommonData->useFastSave(), 0, strm.getFileName()); //ttt2 unify code with Id3V2Rescuer
     vector<const Id3V2Frame*> v (strm.getKnownFrames());
     for (int i = 0; i < cSize(v); ++i)
     {
-        wrt.addNonOwnedFrame(v[i]);
+        const Id3V2Frame* pFrm (v[i]);
+
+        if ('T' == pFrm->m_szName[0])
+        { // text frame
+            try
+            {
+                string s (pFrm->getRawUtf8String());
+                if (!s.empty())
+                {
+                    wrt.addTextFrame(pFrm->m_szName, s);
+                }
+            }
+            catch (const Id3V2Frame::UnsupportedId3V2Frame&)
+            { // add unchanged
+                wrt.addNonOwnedFrame(pFrm);
+            }
+        }
+        else if (0 == strcmp(KnownFrames::LBL_IMAGE(), pFrm->m_szName))
+        {
+            CB_ASSERT (Id3V2Frame::NO_APIC != pFrm->m_eApicStatus);
+            if (Id3V2Frame::COVER == pFrm->m_eApicStatus || Id3V2Frame::NON_COVER == pFrm->m_eApicStatus) // not sure about link; OTOH going to the tab editor will get rid of links, so we do it here as well
+            {
+                wrt.addNonOwnedFrame(pFrm);
+            }
+        }
+        else if (pFrm->m_nMemDataSize > 0)
+        {
+            wrt.addNonOwnedFrame(pFrm);
+        }
     }
 
     TagTimestamp time (strm.getTime());
@@ -129,7 +157,7 @@ bool Id3V2Cleaner::processId3V2Stream(Id3V2StreamBase& strm, ofstream_utf8& out)
 
 
 
-//ttt1 perhaps if there's only one APIC with some unsupported type convert it to "front cover"
+
 
 
 // APIC: if it can be loaded or is unsupported - keep; on error - dump
@@ -137,36 +165,33 @@ bool Id3V2Cleaner::processId3V2Stream(Id3V2StreamBase& strm, ofstream_utf8& out)
 // rewrite text frames to discard null terminators
 bool Id3V2Rescuer::processId3V2Stream(Id3V2StreamBase& strm, ofstream_utf8* pOut) // nothing gets written if pOut is 0
 {
-    Id3V230StreamWriter wrt (m_pCommonData->m_bKeepOneValidImg, m_pCommonData->useFastSave(), 0);
+    Id3V230StreamWriter wrt (m_pCommonData->m_bKeepOneValidImg, m_pCommonData->useFastSave(), 0, strm.getFileName());
 
     const vector<Id3V2Frame*>& vpFrames (strm.getFrames());
 
     for (int i = 0, n = cSize(vpFrames); i < n; ++i)
     {
         const Id3V2Frame* pFrm (vpFrames[i]);
-        Id3V2FrameDataLoader wrp (*pFrm);
 
-        if ('T' == pFrm->m_szName[0] && KnownFrames::getKnownFrames().count(pFrm->m_szName) > 0)
+        if ('T' == pFrm->m_szName[0])
         { // text frame
-            bool bDiscard (false);
             try
             {
-                string s (pFrm->getUtf8String());
-                bDiscard = s.empty();
+                string s (pFrm->getRawUtf8String());
+                if (!s.empty())
+                {
+                    wrt.addTextFrame(pFrm->m_szName, s);
+                }
             }
             catch (const Id3V2Frame::UnsupportedId3V2Frame&)
-            { // !!! nothing to be done here; the frame will get added unchanged
-            }
-
-            if (!bDiscard)
-            {
+            { // add unchanged
                 wrt.addNonOwnedFrame(pFrm);
             }
         }
         else if (0 == strcmp(KnownFrames::LBL_IMAGE(), pFrm->m_szName))
         {
             CB_ASSERT (Id3V2Frame::NO_APIC != pFrm->m_eApicStatus);
-            if (Id3V2Frame::ERROR != pFrm->m_eApicStatus)
+            if (Id3V2Frame::COVER == pFrm->m_eApicStatus || Id3V2Frame::NON_COVER == pFrm->m_eApicStatus) // not sure about link; OTOH going to the tab editor will get rid of links, so we do it here as well
             {
                 wrt.addNonOwnedFrame(pFrm);
             }
@@ -185,7 +210,7 @@ bool Id3V2Rescuer::processId3V2Stream(Id3V2StreamBase& strm, ofstream_utf8* pOut
     return !wrt.contentEqualTo(&strm);
 }
 
-
+//ttt2 consider empty frames; or identifying valid id3v2 frames instead of stopping at first error, also looking in next unknown streams;
 
 /*override*/ Transformation::Result Id3V2Rescuer::apply(const Mp3Handler& h, const TransfConfig& transfConfig, const std::string& strOrigSrcName, std::string& strTempName)
 {
@@ -253,20 +278,22 @@ e1:
             }
             else if (0 != pBrk && pBrk->getBaseName() == Id3V230Stream::getClassDisplayName())
             {
+                in.seekg(pBrk->getPos());
                 NoteColl notes (20);
                 StringWrp fileName (h.getName());
                 Id3V230Stream strm (0, notes, in, &fileName, Id3V230Stream::ACCEPT_BROKEN);
-                Id3V230StreamWriter wrt (m_pCommonData->m_bKeepOneValidImg, m_pCommonData->useFastSave(), &strm);
+                Id3V230StreamWriter wrt (m_pCommonData->m_bKeepOneValidImg, m_pCommonData->useFastSave(), &strm, h.getName());
                 wrt.write(out);
                 bChanged = true;
                 bRecall = true; // !!! now we read whatever frames are available from a broken stream, next time we check for empty or otherwise invalid frames
             }
             else if (0 != pBrk && pBrk->getBaseName() == Id3V240Stream::getClassDisplayName())
             {
+                in.seekg(pBrk->getPos());
                 NoteColl notes (20);
                 StringWrp fileName (h.getName());
-                Id3V240Stream strm (0, notes, in, &fileName, Id3V230Stream::ACCEPT_BROKEN);
-                Id3V230StreamWriter wrt (m_pCommonData->m_bKeepOneValidImg, m_pCommonData->useFastSave(), &strm); //ttt1 if useFastSave is true there should probably be an automatic reload; OTOH we may want to delay until more transforms are applied, so probably it's OK as is
+                Id3V240Stream strm (0, notes, in, &fileName, Id3V240Stream::ACCEPT_BROKEN);
+                Id3V230StreamWriter wrt (m_pCommonData->m_bKeepOneValidImg, m_pCommonData->useFastSave(), &strm, h.getName()); //ttt2 if useFastSave is true there should probably be an automatic reload; OTOH we may want to delay until more transforms are applied, so probably it's OK as is
                 wrt.write(out);
                 bChanged = true;
                 bRecall = true;
@@ -295,10 +322,21 @@ e1:
 //========================================================================================================================
 
 
+/*override*/ const char* Id3V2UnicodeTransformer::getVisibleActionName() const
+{
+    string strActionName (string("Convert non-ASCII ID3V2 text frames to Unicode assuming codepage ") + m_pCommonData->m_pCodec->name().constData());
+    if (strActionName != m_strActionName)
+    {
+        m_strActionName = strActionName; // to make sure that pointer comparisons still work (though they should probably be replaced by string comparisons) //ttt2 replace ptr comparisons
+    }
+    return m_strActionName.c_str();
+}
+
+
 
 void Id3V2UnicodeTransformer::processId3V2Stream(Id3V2StreamBase& strm, ofstream_utf8& out)
 {
-    Id3V230StreamWriter wrt (m_pCommonData->m_bKeepOneValidImg, m_pCommonData->useFastSave(), 0);
+    Id3V230StreamWriter wrt (m_pCommonData->m_bKeepOneValidImg, m_pCommonData->useFastSave(), 0, strm.getFileName());
 
     const vector<Id3V2Frame*>& vpFrames (strm.getFrames());
 
@@ -398,7 +436,7 @@ struct PLPAS
         {
             QChar c (i);
             QString q; q += c;
-            cout << i << " " << q.toUtf8().data() << " " << (int)c.category() << endl;
+            cout << i << " " << q.toUtf8().constData() << " " << (int)c.category() << endl;
         }
     }
 };
@@ -407,74 +445,23 @@ struct PLPAS
 PLPAS llpwwe;
 */
 
-static QString getCaseConv(const QString& s, CommonData::Case eCase)
+
+
+/*override*/ const char* Id3V2CaseTransformer::getVisibleActionName() const
 {
-/*
-    lNames << "Lower case: first part. second part.";
-    lNames << "Upper case: FIRST PART. SECOND PART.";
-    lNames << "Title case: First Part. Second Part.";
-    lNames << "Phrase case: First part. Second part.";
-*/
-    switch(eCase)
+    string strActionName (string("Change case for ID3V2 text frames: Artists - ") + getCaseAsStr(m_pCommonData->m_eCaseForArtists) + "; Others - " + getCaseAsStr(m_pCommonData->m_eCaseForOthers));
+
+    if (strActionName != m_strActionName)
     {
-    case CommonData::LOWER: return s.toLower();
-
-    case CommonData::UPPER: return s.toUpper();
-
-    case CommonData::TITLE:
-        {
-            int n (s.size());
-            QString res;
-            bool bWhitesp (true);
-            for (int i = 0; i < n; ++i)
-            {
-                const QChar& qc (s[i]);
-                if (bWhitesp)
-                {
-                    res += qc.toUpper();
-                }
-                else
-                {
-                    res += qc.toLower();
-                }
-
-                bWhitesp = qc.isSpace() || qc == '.';
-            }
-            return res;
-        }
-
-    case CommonData::PHRASE:
-        {
-            int n (s.size());
-            QString res;
-            bool bPer (true);
-            for (int i = 0; i < n; ++i)
-            {
-                const QChar& qc (s[i]);
-                if (bPer)
-                {
-                    res += qc.toUpper();
-                }
-                else
-                {
-                    res += qc.toLower();
-                }
-
-                if (!qc.isSpace()) { bPer = (qc == '.'); }
-            }
-            return res;
-        }
+        m_strActionName = strActionName; // to make sure that pointer comparisons still work (though they should probably be replaced by string comparisons) //ttt2 replace ptr comparisons
     }
-
-    CB_ASSERT (false);
+    return m_strActionName.c_str();
 }
 
 
-
-
 bool Id3V2CaseTransformer::processId3V2Stream(Id3V2StreamBase& strm, ofstream_utf8& out)
 {
-    Id3V230StreamWriter wrt (m_pCommonData->m_bKeepOneValidImg, m_pCommonData->useFastSave(), &strm);
+    Id3V230StreamWriter wrt (m_pCommonData->m_bKeepOneValidImg, m_pCommonData->useFastSave(), &strm, strm.getFileName());
 
     {
         const Id3V2Frame* pFrm (strm.getFrame(KnownFrames::LBL_ARTIST()));
@@ -594,7 +581,7 @@ bool Id3V2CaseTransformer::processId3V2Stream(Id3V2StreamBase& strm, ofstream_ut
 
 bool Id3V1ToId3V2Copier::processId3V2Stream(Id3V2StreamBase& strm, ofstream_utf8& out, Id3V1Stream* pId3V1Stream)
 {
-    Id3V230StreamWriter wrt (m_pCommonData->m_bKeepOneValidImg, m_pCommonData->useFastSave(), &strm);
+    Id3V230StreamWriter wrt (m_pCommonData->m_bKeepOneValidImg, m_pCommonData->useFastSave(), &strm, strm.getFileName());
 
     if (strm.getTitle().empty())
     {
@@ -666,7 +653,7 @@ bool Id3V1ToId3V2Copier::processId3V2Stream(Id3V2StreamBase& strm, ofstream_utf8
 
     if (!bId3V2Found)
     {
-        Id3V230StreamWriter wrt (m_pCommonData->m_bKeepOneValidImg, m_pCommonData->useFastSave(), 0);
+        Id3V230StreamWriter wrt (m_pCommonData->m_bKeepOneValidImg, m_pCommonData->useFastSave(), 0, h.getName());
 
         {
             string s (pId3V1Stream->getTitle());
@@ -724,7 +711,7 @@ bool Id3V1ToId3V2Copier::processId3V2Stream(Id3V2StreamBase& strm, ofstream_utf8
     { // temp
         transfConfig.getTempName(strOrigSrcName, getActionName(), strTempName);
         ofstream_utf8 out (strTempName.c_str(), ios::binary);
-        in.seekg(0); // ttt1 see if still needed
+        in.seekg(0); // ttt2 doesn't seem needed; search for similar cases and perhaps remove them all
 
         for (int i = 0, n = cSize(vpStreams); i < n; ++i)
         {
@@ -781,7 +768,7 @@ bool Id3V1ToId3V2Copier::processId3V2Stream(Id3V2StreamBase& strm, ofstream_utf8
     if (beginsWith(strArtist, strComp + " [") && endsWith(strArtist, "]")) { return NOT_CHANGED; }
 
     { // temp
-        Id3V230StreamWriter wrt (m_pCommonData->m_bKeepOneValidImg, m_pCommonData->useFastSave(), pId3V2);
+        Id3V230StreamWriter wrt (m_pCommonData->m_bKeepOneValidImg, m_pCommonData->useFastSave(), pId3V2, h.getName());
         wrt.addTextFrame(KnownFrames::LBL_ARTIST(), strComp + " [" + strArtist + "]");
 
         ifstream_utf8 in (h.getName().c_str(), ios::binary);
@@ -831,7 +818,7 @@ bool Id3V1ToId3V2Copier::processId3V2Stream(Id3V2StreamBase& strm, ofstream_utf8
     if (!(beginsWith(strArtist, strComp + " [") && endsWith(strArtist, "]"))) { return NOT_CHANGED; }
 
     { // temp
-        Id3V230StreamWriter wrt (m_pCommonData->m_bKeepOneValidImg, m_pCommonData->useFastSave(), pId3V2);
+        Id3V230StreamWriter wrt (m_pCommonData->m_bKeepOneValidImg, m_pCommonData->useFastSave(), pId3V2, h.getName());
         //wrt.addTextFrame(KnownFrames::LBL_ARTIST(), strComp + " [" + strArtist + "]");
         wrt.addTextFrame(KnownFrames::LBL_ARTIST(), strArtist.substr(strComp.size() + 2, strArtist.size() - strComp.size() - 3));
 
@@ -886,7 +873,7 @@ bool Id3V1ToId3V2Copier::processId3V2Stream(Id3V2StreamBase& strm, ofstream_utf8
     if (strComp == strExistingComp) { return NOT_CHANGED; }
 
     { // temp
-        Id3V230StreamWriter wrt (m_pCommonData->m_bKeepOneValidImg, m_pCommonData->useFastSave(), pId3V2);
+        Id3V230StreamWriter wrt (m_pCommonData->m_bKeepOneValidImg, m_pCommonData->useFastSave(), pId3V2, h.getName());
         wrt.addTextFrame(KnownFrames::LBL_COMPOSER(), strComp);
 
         ifstream_utf8 in (h.getName().c_str(), ios::binary);
@@ -916,8 +903,10 @@ bool Id3V1ToId3V2Copier::processId3V2Stream(Id3V2StreamBase& strm, ofstream_utf8
 
 
 
-/*override*/ Transformation::Result SmallerImageRemover::apply(const Mp3Handler& h, const TransfConfig& transfConfig, const std::string& strOrigSrcName, std::string& strTempName)
+/*override*/ Transformation::Result SmallerImageRemover::apply(const Mp3Handler& h, const TransfConfig& transfConfig, const std::string& strOrigSrcName, std::string& strTempName) //ttt1 recompress large images
 {
+    LAST_STEP("SmallerImageRemover::apply() " + h.getName());
+
     const vector<DataStream*>& vpStreams (h.getStreams());
     Id3V2StreamBase* pId3V2 (0);
 
@@ -938,7 +927,8 @@ bool Id3V1ToId3V2Copier::processId3V2Stream(Id3V2StreamBase& strm, ofstream_utf8
     for (int i = 0; i < cSize(vpFrames); ++i)
     {
         const Id3V2Frame* p (vpFrames[i]);
-        if (0 == strcmp(p->m_szName, KnownFrames::LBL_IMAGE()))
+        //if (0 == strcmp(p->m_szName, KnownFrames::LBL_IMAGE()))
+        if (Id3V2Frame::COVER == p->m_eApicStatus || Id3V2Frame::NON_COVER == p->m_eApicStatus)
         {
             ++nPicCnt;
             if (0 == pLargestPic || pLargestPic->m_nImgSize < p->m_nImgSize)
@@ -948,12 +938,12 @@ bool Id3V1ToId3V2Copier::processId3V2Stream(Id3V2StreamBase& strm, ofstream_utf8
         }
     }
 
-    if (0 == nPicCnt || (1 == nPicCnt && Id3V2Frame::COVER == pLargestPic->m_nPictureType)) { return NOT_CHANGED; }
+    if (0 == nPicCnt || (1 == nPicCnt && Id3V2Frame::PT_COVER == pLargestPic->m_nPictureType)) { return NOT_CHANGED; }
 
 
 
     { // temp
-        Id3V230StreamWriter wrt (Id3V230StreamWriter::KEEP_ONE_VALID_IMG, m_pCommonData->useFastSave(), pId3V2);
+        Id3V230StreamWriter wrt (Id3V230StreamWriter::KEEP_ONE_VALID_IMG, m_pCommonData->useFastSave(), pId3V2, h.getName());
 
         wrt.removeFrames(KnownFrames::LBL_IMAGE());
 
@@ -962,7 +952,7 @@ bool Id3V1ToId3V2Copier::processId3V2Stream(Id3V2StreamBase& strm, ofstream_utf8
         copy (ldr.getData(), ldr.getData() + pLargestPic->m_nMemDataSize, back_inserter(v));
 
         //wrt.addBinaryFrame(KnownFrames::LBL_IMAGE(), v);
-        wrt.addImage(v);
+        wrt.addImg(v);
 
         ifstream_utf8 in (h.getName().c_str(), ios::binary);
         transfConfig.getTempName(strOrigSrcName, getActionName(), strTempName);
@@ -1017,7 +1007,7 @@ bool Id3V1ToId3V2Copier::processId3V2Stream(Id3V2StreamBase& strm, ofstream_utf8
     if (nExtraSize <= nOldPaddingSize) { return NOT_CHANGED; }
 
     { // temp
-        Id3V230StreamWriter wrt (m_pCommonData->m_bKeepOneValidImg, m_pCommonData->useFastSave(), pId3V2);
+        Id3V230StreamWriter wrt (m_pCommonData->m_bKeepOneValidImg, m_pCommonData->useFastSave(), pId3V2, h.getName());
 
         ifstream_utf8 in (h.getName().c_str(), ios::binary);
         transfConfig.getTempName(strOrigSrcName, getActionName(), strTempName);
@@ -1064,7 +1054,7 @@ bool Id3V1ToId3V2Copier::processId3V2Stream(Id3V2StreamBase& strm, ofstream_utf8
     if (pId3V2->getPaddingSize() < Id3V230StreamWriter::DEFAULT_EXTRA_SPACE + 512) { return NOT_CHANGED; }
 
     { // temp
-        Id3V230StreamWriter wrt (m_pCommonData->m_bKeepOneValidImg, m_pCommonData->useFastSave(), pId3V2);
+        Id3V230StreamWriter wrt (m_pCommonData->m_bKeepOneValidImg, m_pCommonData->useFastSave(), pId3V2, h.getName());
 
         ifstream_utf8 in (h.getName().c_str(), ios::binary);
         transfConfig.getTempName(strOrigSrcName, getActionName(), strTempName);
@@ -1094,7 +1084,7 @@ bool Id3V1ToId3V2Copier::processId3V2Stream(Id3V2StreamBase& strm, ofstream_utf8
 //========================================================================================================================
 
 //ttt2 perhaps be able to extract composer even when the field is empty, if artist is "composer [artist]", but doesn't look too useful
-//ttt1 perhaps something to discard invalid ID3V2 frames, especially invalid pictures
+
 
 
 //ttt2 warn in config if user enables fast save and then hides Id3V2Compactor
