@@ -23,11 +23,14 @@
 
 #include  <algorithm>
 
+#include  <boost/static_assert.hpp>
+
 #include  "Id3V230Stream.h"
 
 #include  "Id3V240Stream.h"
 #include  "Helpers.h"
 #include  "Id3Transf.h"
+#include  "CommonData.h"
 
 
 
@@ -56,7 +59,7 @@ Id3V230Frame::Id3V230Frame(NoteColl& notes, istream& in, streampos pos, bool bHa
     if (0 == bfr[0])
     { // padding
         //m_nMemDataSize = -1; // !!! not needed; the constructor makes it -1
-        m_nDiskDataSize = -1;
+        //m_nDiskDataSize = -1; // !!! not needed; the constructor makes it -1
         m_szName[0] = 0;
         return;
     }
@@ -65,6 +68,8 @@ Id3V230Frame::Id3V230Frame(NoteColl& notes, istream& in, streampos pos, bool bHa
     m_szName[4] = 0;
     m_nMemDataSize = (p[4] << 24) + (p[5] << 16) + (p[6] << 8) + (p[7] << 0);
 
+    MP3_CHECK (m_nMemDataSize >= 0 && m_nMemDataSize < 5000000, pos, id3v230CantReadFrame, StreamIsBroken(Id3V230Stream::getClassDisplayName(), "Broken ID3V2.3.0 tag."));
+
     {
         char c (m_szName[0]);
         MP3_CHECK (c >= 'A' && c <= 'Z', pos, id3v2InvalidName, StreamIsBroken(Id3V230Stream::getClassDisplayName(), "ID3V2.3.0 tag containing a frame with an invalid name: " + getReadableName() + "."));
@@ -86,10 +91,13 @@ Id3V230Frame::Id3V230Frame(NoteColl& notes, istream& in, streampos pos, bool bHa
     m_vcData.resize(m_nMemDataSize);
     int nContentBytesSkipped (0);
     nRead = 0;
-    nRead = readID3V2(bHasUnsynch, in, &m_vcData[0], m_nMemDataSize, posNext, nContentBytesSkipped);
+    if (m_nMemDataSize > 0)
+    {
+        nRead = readID3V2(bHasUnsynch, in, &m_vcData[0], m_nMemDataSize, posNext, nContentBytesSkipped);
+    }
     if (m_nMemDataSize != nRead)
     {
-        vector<char>().swap(m_vcData);;
+        vector<char>().swap(m_vcData);
         MP3_THROW (pos, id3v2FrameTooShort, StreamIsBroken(Id3V230Stream::getClassDisplayName(), "Truncated ID3V2.3.0 tag."));
     }
 
@@ -101,6 +109,40 @@ Id3V230Frame::Id3V230Frame(NoteColl& notes, istream& in, streampos pos, bool bHa
         try
         {
             getUtf8String();
+
+            Id3V2FrameDataLoader wrp (*this);
+            const char* pData (wrp.getData());
+
+            if ('T' == m_szName[0])
+            {
+                if (!isTxxx() && 0 == m_nMemDataSize)
+                { // this is really invalid; text frames must have at least a byte; however, by doing these we make sure that an empty (i.e. having a single byte, for text encoding) frame gets copied if the tag is edited;
+                    m_vcData.clear();
+                    m_vcData.push_back(0);
+                    m_nMemDataSize = cSize(m_vcData);
+                    MP3_NOTE_D (pos, id3v2EmptyTextFrame, Notes::id3v2EmptyTextFrame().getDescription() + string(" (Frame:") + m_szName + ")");
+                }
+                else if (isTxxx() && m_nMemDataSize <= 1)
+                { // this is really invalid; text frames must have at least a byte; however, by doing these we make sure that an empty (i.e. having a single byte, for text encoding) frame gets copied if the tag is edited;
+                    m_vcData.clear();
+                    static const char* szInvalid ("INVALID");
+                    m_vcData.push_back(0); // latin1
+                    m_vcData.insert(m_vcData.end(), szInvalid, szInvalid + strlen(szInvalid)); // description
+                    m_vcData.push_back(0); // terminator
+                    m_vcData.insert(m_vcData.end(), szInvalid, szInvalid + strlen(szInvalid)); // value
+                    m_nMemDataSize = cSize(m_vcData);
+                    MP3_NOTE_D (pos, id3v2EmptyTextFrame, Notes::id3v2EmptyTextFrame().getDescription() + string(" (Frame:") + m_szName + ")"); // ttt2 actually TXXX is not text
+                }
+                else
+                {
+                    if (3 == pData[0])
+                    {
+                        //MP3_NOTE (pos, id3v230UsesUtf8);
+                        MP3_NOTE_D (pos, id3v230UsesUtf8, Notes::id3v230UsesUtf8().getDescription() + string(" (Frame:") + m_szName + ")"); // perhaps drop the frame name if too many such notes get generated
+                    }
+                }
+                //ttt2 add check for embedded 0 for TXXX, to split the value into descr and val
+            }
         }
         catch (const NotId3V2Frame&)
         {
@@ -114,7 +156,7 @@ Id3V230Frame::Id3V230Frame(NoteColl& notes, istream& in, streampos pos, bool bHa
     catch (const std::bad_alloc&) { throw; }
     catch (...)
     {
-        vector<char>().swap(m_vcData);;
+        vector<char>().swap(m_vcData);
         throw;
     }
 
@@ -153,9 +195,9 @@ Id3V230Frame::Id3V230Frame(const std::string& strName, vector<char>& vcData) : I
 
 
 
-// assumes Latin1, converts to UTF8; returns "<non-text value>" for non-text strings; replaces '\0' with ' ' inside tags; //ttt2 see about implications; "Momma Cried" has a tag called TXXX whose value contains a '\0': "RATING\01";
-// whitespaces at the end of the string are removed; not sure if this is how it should be, though
-string Id3V230Frame::getUtf8String() const
+// may return multiple null characters; it's the job of getUtf8String() to deal with them;
+// chars after the first null are considered comments (or after the second null, for TXXX);
+string Id3V230Frame::getUtf8StringImpl() const
 {
     if ('T' != m_szName[0])
     {
@@ -167,21 +209,18 @@ string Id3V230Frame::getUtf8String() const
     // or perhaps forget these and throw exceptions that have error messages, catch them and log/show dialogs
     // 2008.07.12 - actually all these proposals don't seem to work well: the callers don't catch the exceptions thrown here and wouldn't know what to do with them; there's no good place to display the errors (the end user isn't supposed to look at logs); it seems better to not throw, but return a string describing the problem;
     // 2008.07.12 - on a second thought - throw but call this on the constructor, where it can be logged properly; if it worked on the constructor it should work later too
-    CB_CHECK1 (m_nMemDataSize > 0, NotId3V2Frame());
+    CB_CHECK1 (m_nMemDataSize > 0, NotId3V2Frame()); //ttt2 perhaps other excp
     Id3V2FrameDataLoader wrp (*this);
     const char* pData (wrp.getData());
+    string s;
+
     if (0 == pData[0])
     { // Latin-1
-        string s;
         for (int i = 1; i < m_nMemDataSize; ++i)
         {
             unsigned char c (pData[i]);
-            if (0 == c)
-            {
-                s += ' ';
-            }
-            else if (c < 128)
-            {
+            if (c < 128)
+            { // !!! 0 is OK
                 s += char(c);
             }
             else
@@ -193,17 +232,22 @@ string Id3V230Frame::getUtf8String() const
                 s += char(c2);
             }
         }
-        rtrim(s);
-        return s;
     }
-
-    if (1 == pData[0])
+    else if (1 == pData[0])
     {
-        return utf8FromBomUtf16(pData + 1, m_nMemDataSize - 1);
+        s = utf8FromBomUtf16(pData + 1, m_nMemDataSize - 1);
+    }
+    else if (3 == pData[0]) // not valid, but used by some tools; a note will get generated in this case
+    {
+        s = string(pData + 1, m_nMemDataSize - 1);
+    }
+    else
+    {
+        // pData[0] has an invalid value
+        CB_THROW1 (NotId3V2Frame());
     }
 
-    // pData[0] has an invalid value
-    CB_THROW1 (NotId3V2Frame());
+    return s;
 }
 
 
@@ -233,7 +277,7 @@ Id3V230Stream::Id3V230Stream(int nIndex, NoteColl& notes, istream& in, StringWrp
     MP3_CHECK_T (3 == bfr[3] && 0 == bfr[4], pos, "Invalid ID3V2.3.0 tag. Invalid ID3V2.3.0 header.", NotId3V2());
     m_nTotalSize = getId3V2Size (bfr);
     m_cFlags = bfr[5];
-    MP3_CHECK (0 == (m_cFlags & 0x7f), pos, id3v2UnsuppFlag, StreamIsUnsupported(Id3V230Stream::getClassDisplayName(), "ID3V2 tag with unsupported flag.")); //ttt1 review, support
+    MP3_CHECK (0 == (m_cFlags & 0x7f), pos, id3v2UnsuppFlag, StreamIsUnsupported(Id3V230Stream::getClassDisplayName(), "ID3V2 tag with unsupported flag.")); //ttt2 review, support
 
     streampos posNext (pos);
     posNext += m_nTotalSize;
@@ -248,7 +292,7 @@ Id3V230Stream::Id3V230Stream(int nIndex, NoteColl& notes, istream& in, StringWrp
             Id3V230Frame* p (new Id3V230Frame(notes, in, pos, hasUnsynch(), posNext, m_pFileName));
             bHasLatin1NonAscii = p->m_bHasLatin1NonAscii || bHasLatin1NonAscii;
             if (-1 == p->m_nMemDataSize)
-            { // it encountered zeroes, which signals the beginning of padding
+            { // it encountered zeroes, which signals the beginning of padding //ttt2 should check that there's no garbage after the first zero
                 m_nPaddingSize = posNext - pos;
                 delete p;
                 break;
@@ -262,6 +306,7 @@ Id3V230Stream::Id3V230Stream(int nIndex, NoteColl& notes, istream& in, StringWrp
     {
         if (bAcceptBroken)
         {
+            preparePicture(notes);
             return;
         }
 
@@ -292,8 +337,22 @@ Id3V230Stream::Id3V230Stream(int nIndex, NoteColl& notes, istream& in, StringWrp
         MP3_NOTE (m_pos + (getSize() - m_nPaddingSize), id3v2PaddingTooLarge);
     }
 
+    if (m_vpFrames.empty())
+    {
+        MP3_NOTE (m_pos, id3v2EmptyTag);
+    }
+
     MP3_TRACE (m_pos, "Id3V230Stream built.");
 
+    {
+        pos = m_pos;
+        pos += m_nTotalSize - 1;
+        in.seekg(pos);
+        char c;
+
+        MP3_CHECK (1 == read(in, &c, 1), m_pos, id3v230CantReadFrame, NotId3V2());
+    }
+
     rst.setOk();
 }
 
@@ -311,6 +370,7 @@ Id3V230Stream::Id3V230Stream(int nIndex, NoteColl& notes, istream& in, StringWrp
     case ALBUM:
     case RATING:
     case COMPOSER:
+    case VARIOUS_ARTISTS:
         return READ_ONLY;
 
     default:
@@ -345,7 +405,7 @@ Id3V230StreamWriter::~Id3V230StreamWriter()
 
 
 
-Id3V230StreamWriter::Id3V230StreamWriter(bool bKeepOneValidImg, bool bFastSave, Id3V2StreamBase* p) : m_bKeepOneValidImg(bKeepOneValidImg), m_bFastSave(bFastSave)
+Id3V230StreamWriter::Id3V230StreamWriter(bool bKeepOneValidImg, bool bFastSave, Id3V2StreamBase* p, const std::string& strDebugFileName) : m_bKeepOneValidImg(bKeepOneValidImg), m_bFastSave(bFastSave), m_strDebugFileName(strDebugFileName)
 {
     if (0 != p)
     {
@@ -356,7 +416,7 @@ Id3V230StreamWriter::Id3V230StreamWriter(bool bKeepOneValidImg, bool bFastSave,
             for (int i = 0; i < cSize(p->getFrames()); ++i)
             {
                 const Id3V2Frame* q (p->getFrames()[i]);
-                CB_ASSERT ((0 == strcmp(q->m_szName, KnownFrames::LBL_IMAGE())) ^ (Id3V2Frame::NO_APIC == q->m_eApicStatus));
+                CB_ASSERT1 ((0 == strcmp(q->m_szName, KnownFrames::LBL_IMAGE())) ^ (Id3V2Frame::NO_APIC == q->m_eApicStatus), m_strDebugFileName);
                 if (Id3V2Frame::NO_APIC == q->m_eApicStatus)
                 {
                     bool bCopyFrame (true);
@@ -368,7 +428,7 @@ Id3V230StreamWriter::Id3V230StreamWriter(bool bKeepOneValidImg, bool bFastSave,
                         if (3 == pData[0])
                         { // UTF-8
                             bCopyFrame = false;
-                            addTextFrame(q->m_szName, q->getUtf8String());
+                            addTextFrame(q->m_szName, q->getRawUtf8String());
                         }
                     }
 
@@ -379,7 +439,7 @@ Id3V230StreamWriter::Id3V230StreamWriter(bool bKeepOneValidImg, bool bFastSave,
                 }
                 else
                 {
-                    if ((0 == pPic && q->m_eApicStatus > Id3V2Frame::ERROR) ||
+                    if ((0 == pPic && q->m_eApicStatus > Id3V2Frame::ERR) ||
                         (0 != pPic && pPic->m_eApicStatus < q->m_eApicStatus))
                     {
                         pPic = q;
@@ -425,8 +485,77 @@ void Id3V230StreamWriter::setRecTime(const TagTimestamp& time)
 }
 
 
+// this only changes the frames that correspond to the active settings in the configuration;
+// if WMP handling is disabled, TPE2 is left untouched; if WMP handling is enabled, TPE2 is either removed or set to "Various Artists", based on "b"
+// if iTunes handling is disabled, TCON is left untouched; if WMP handling is enabled, TCON is either removed or set to "1", based on "b"
+void Id3V230StreamWriter::setVariousArtists(bool b)
+{
+    const CommonData* pCommonData (getCommonData());
 
-// strVal is UTF8; the frame will use ASCII if possible and UTF16 otherwise (so if there's a char with a code above 127, UTF16 gets used, to avoid codepage issues for codes between 128 and 255); nothing is added if strVal is empty;
+    if (pCommonData->m_bWmpVarArtists)
+    {
+        removeFrames(KnownFrames::LBL_WMP_VAR_ART());
+        if (b)
+        {
+            addTextFrame(KnownFrames::LBL_WMP_VAR_ART(), "Various Artists");
+        }
+    }
+
+    if (pCommonData->m_bItunesVarArtists)
+    {
+        removeFrames(KnownFrames::LBL_ITUNES_VAR_ART());
+        if (b)
+        {
+            addTextFrame(KnownFrames::LBL_ITUNES_VAR_ART(), "1");
+        }
+    }
+}
+
+#ifdef MASWDEFWDWDWDW
+namespace
+{
+    void view(QString q)
+    {
+        qDebug("str=%s, sz=%d", q.toUtf8().data(), q.size());
+
+        {
+            qDebug("utf16()");
+            const ushort* pUtf16 (q.utf16());
+            for (int i = 0; 0 != pUtf16[i]; ++i)
+            {
+                qDebug("    %d", int(pUtf16[i]));
+            }
+        }
+
+        {
+            qDebug("toUcs4()");
+            QVector<uint> v (q.toUcs4());
+            for (int i = 0; i < v.size(); ++i)
+            {
+                qDebug("    %d", int(v[i]));
+            }
+        }
+    }
+
+    void testUnicode() //ttt2 retest this; for now it looks like everything is truncated to 16 bits
+    {
+        QString q;
+        q += QChar(49);
+        view(q);
+        q += QChar(int(65536 + 65));
+        q += QChar(int(65536 + 650));
+        q += QChar(int(65536 + 6500));
+        q += QChar(int(65536 + 35000));
+        view(q);
+        //q += QChar(int(0xd801)); // surrogate
+        //view(q);
+    }
+}
+#endif
+
+BOOST_STATIC_ASSERT(2 == sizeof(ushort));
+
+// strVal is UTF8; the frame will use ASCII if possible and UTF16 otherwise (so if there's a char with a code above 127, UTF16 gets used, to avoid codepage issues for codes between 128 and 255); nothing is added if strVal is empty; all zeroes are saved and not considered terminators;
 void Id3V230StreamWriter::addTextFrame(const std::string& strName, const std::string& strVal)
 {
 //http://www.id3.org/id3v2.3.0#head-1a37d4a15deafc294208ccfde950f77e47000bca
@@ -455,10 +584,11 @@ void Id3V230StreamWriter::addTextFrame(const std::string& strName, const std::st
     else
     {
         QString s (convStr(strVal));
-        const ushort* pUtf16 (s.utf16()); //ttt3 assumes short is 16bit
-        int nUtf16Size (0);
-        while (0 != pUtf16[nUtf16Size++]) {} // there is probably some strlen equivalent, but wcslen isn't that, because it uses wchar_t instead of ushort, which on Unix is usually 4-byte
-        --nUtf16Size;
+        const ushort* pUtf16 (s.utf16());
+        //int nUtf16Size (0);
+        //while (0 != pUtf16[nUtf16Size++]) {} // there is probably some strlen equivalent, but wcslen isn't that, because it uses wchar_t instead of ushort, which on Unix is usually 4-byte
+        //--nUtf16Size;
+        int nUtf16Size (s.size()); //ttt2 not quite right, but it seems that QString truncates Unicode to 16 bits anyway, in which case it's fine; see testUnicode(), above
         nSize = 2*nUtf16Size + 3; // "text encoding" byte, BOM, no null terminator
         vcData.resize(nSize);
         vcData[0] = 1;
@@ -469,8 +599,8 @@ void Id3V230StreamWriter::addTextFrame(const std::string& strName, const std::st
         int nOrder (0);
 #endif
 
-        vcData[2 - nOrder] = 0xff;
-        vcData[1 + nOrder] = 0xfe;
+        vcData[2 - nOrder] = char(0xff);
+        vcData[1 + nOrder] = char(0xfe);
 
         const char* pcUtf16 (reinterpret_cast<const char*>(pUtf16));
         copy(pcUtf16, pcUtf16 + nSize - 3, &vcData[0] + 3);
@@ -486,7 +616,7 @@ void Id3V230StreamWriter::addTextFrame(const std::string& strName, const std::st
 void Id3V230StreamWriter::addBinaryFrame(const std::string& strName, vector<char>& vcData)
 {
     Id3V230Frame* p (new Id3V230Frame(strName, vcData));
-    CB_ASSERT (0 != strcmp(KnownFrames::LBL_IMAGE(), p->m_szName));
+    CB_ASSERT1 (0 != strcmp(KnownFrames::LBL_IMAGE(), p->m_szName), m_strDebugFileName);
     addNonOwnedFrame(p);
     m_vpOwnFrames.push_back(p);
 }
@@ -498,25 +628,26 @@ void Id3V230StreamWriter::addBinaryFrame(const std::string& strName, vector<char
 // the image type is ignored; images are always added as cover;
 // if there is an APIC frame with the same image, it is removed (it doesn't matter if it has different type, description ...); // ttt2 description should be counted too, if used
 // if cover image already exists it is removed;
-void Id3V230StreamWriter::addImage(std::vector<char>& vcData)
+void Id3V230StreamWriter::addImg(std::vector<char>& vcData)
 {
     int n (cSize(vcData));
-    CB_ASSERT (n > 100);
+    CB_ASSERT1 (n > 100, m_strDebugFileName);
     char* p (&vcData[0]);
-    CB_ASSERT (0 == *p || 3 == *p); // text encoding // this should be kept in synch with Id3V2StreamBase::decodeApic()
+    CB_ASSERT1 (0 == *p || 3 == *p, m_strDebugFileName); // text encoding // this should be kept in synch with Id3V2StreamBase::decodeApic() //ttt1 triggered according to mail; might have been caused by SmallerImageRemover::apply() incorrectly assuming that an invalid APIC frame is a large picture; the test using the frame name was replaced after the assert with a test using m_eApicStatus; will have to wait until some MP3 is received that triggered this to be sure
+
     char* q (p + 1);
     for (; q < p + 90 && 0 != *q; ++q) {}
-    CB_ASSERT (0 == *q);
+    CB_ASSERT1 (0 == *q, m_strDebugFileName);
     ++q;
-    *q = Id3V2Frame::COVER;
+    *q = Id3V2Frame::PT_COVER;
     ++q;
     for (; q < p + n && 0 != *q; ++q) {}
-    CB_ASSERT (0 == *q);
+    CB_ASSERT1 (0 == *q, m_strDebugFileName);
     ++q; // now q points to the beginning of the actual image
     int nOffs (q - p);
     int nImgSize (n - nOffs);
 
-    removeFrames(KnownFrames::LBL_IMAGE(), Id3V2Frame::COVER);
+    removeFrames(KnownFrames::LBL_IMAGE(), Id3V2Frame::PT_COVER);
 
     {
 e1:
@@ -529,7 +660,7 @@ e1:
                 if (pFrm->m_nImgSize == nImgSize)
                 {
                     Id3V2FrameDataLoader ldr (*pFrm);
-                    if (0 == memcmp(&vcData[nOffs], ldr.getData() + pFrm->m_nImgOffset, nImgSize)) // !!! related to ImageInfo::operator==() status is ignored in both places //ttt1 review decision to ignore status
+                    if (0 == memcmp(&vcData[nOffs], ldr.getData() + pFrm->m_nImgOffset, nImgSize)) // !!! related to ImageInfo::operator==() status is ignored in both places //ttt2 review decision to ignore status
                     {
                         removeFrames(KnownFrames::LBL_IMAGE(), pFrm->m_nPictureType);
                         goto e1;
@@ -541,7 +672,7 @@ e1:
 
     {
         Id3V230Frame* p (new Id3V230Frame(KnownFrames::LBL_IMAGE(), vcData));
-        p->m_nPictureType = Id3V2Frame::COVER; // probably pointless, because the frame is only used internally and what gets written is vcData, regardless of p->m_nPictureType
+        p->m_nPictureType = Id3V2Frame::PT_COVER; // probably pointless, because the frame is only used internally and what gets written is vcData, regardless of p->m_nPictureType
         addNonOwnedFrame(p);
         m_vpOwnFrames.push_back(p);
     }
@@ -556,7 +687,7 @@ void Id3V230StreamWriter::addNonOwnedFrame(const Id3V2Frame* p)
     if (0 == strcmp(KnownFrames::LBL_IMAGE(), p->m_szName) || !KnownFrames::canHaveDuplicates(p->m_szName))
     {
         removeFrames(p->m_szName, p->m_nPictureType);
-        // m_vpAllFrames.insert(m_vpAllFrames.begin(), p); //ttt1 putting a front cover image after a back cover might not be the best idea; see if it makes sense to sort the frames; (perhaps have something to sort all frames before saving)
+        // m_vpAllFrames.insert(m_vpAllFrames.begin(), p); //ttt2 putting a front cover image after a back cover might not be the best idea; see if it makes sense to sort the frames; (perhaps have something to sort all frames before saving)
     }
 
     if ('T' == p->m_szName[0] && p->m_nMemDataSize > 0 && 0 != dynamic_cast<const Id3V240Frame*>(p))
@@ -565,7 +696,7 @@ void Id3V230StreamWriter::addNonOwnedFrame(const Id3V2Frame* p)
         const char* pData (ldr.getData());
         if (3 == pData[0])
         { // UTF-8
-            addTextFrame(p->m_szName, p->getUtf8String());
+            addTextFrame(p->m_szName, p->getRawUtf8String());
             return;
         }
     }
@@ -573,9 +704,9 @@ void Id3V230StreamWriter::addNonOwnedFrame(const Id3V2Frame* p)
     m_vpAllFrames.push_back(p);
 }
 
-//ttt1 perhaps transform to change image types to Cover
 
-// if multiple frames with the same name exist, they are all removed; asserts that nPictureType is -1 for non-APIC frames
+
+// if multiple frames with the same name exist, they are all removed; asserts that nPictureType is -1 for non-APIC frames; if nPictureType is -1 and strName is APIC, it removes all APIC frames
 void Id3V230StreamWriter::removeFrames(const std::string& strName, int nPictureType /*= -1*/)
 {
     const Id3V2Frame* p;
@@ -583,8 +714,8 @@ void Id3V230StreamWriter::removeFrames(const std::string& strName, int nPictureT
     for (int i = cSize(m_vpAllFrames) - 1; i >= 0; --i)
     {
         p = m_vpAllFrames[i];
-        CB_ASSERT (-1 == nPictureType || KnownFrames::LBL_IMAGE() == strName);
-        if (p->m_szName == strName && (p->m_nPictureType == nPictureType || m_bKeepOneValidImg))
+        CB_ASSERT1 (-1 == nPictureType || KnownFrames::LBL_IMAGE() == strName, m_strDebugFileName);
+        if (p->m_szName == strName && (p->m_nPictureType == nPictureType || m_bKeepOneValidImg || -1 == nPictureType))
         {
             m_vpAllFrames.erase(m_vpAllFrames.begin() + i);
         }
@@ -593,8 +724,8 @@ void Id3V230StreamWriter::removeFrames(const std::string& strName, int nPictureT
     for (int i = cSize(m_vpOwnFrames) - 1; i >= 0; --i)
     {
         p = m_vpOwnFrames[i];
-        CB_ASSERT (-1 == nPictureType || KnownFrames::LBL_IMAGE() == strName);
-        if (p->m_szName == strName && (p->m_nPictureType == nPictureType || m_bKeepOneValidImg))
+        CB_ASSERT1 (-1 == nPictureType || KnownFrames::LBL_IMAGE() == strName, m_strDebugFileName);
+        if (p->m_szName == strName && (p->m_nPictureType == nPictureType || m_bKeepOneValidImg || -1 == nPictureType))
         {
             delete p;
             m_vpOwnFrames.erase(m_vpOwnFrames.begin() + i);
@@ -602,7 +733,7 @@ void Id3V230StreamWriter::removeFrames(const std::string& strName, int nPictureT
     }
 }
 
-//ttt1 after rescanning when note text changed, the refound error was not visible once (couldn't replicate)
+//ttt2 after rescanning when note text changed, the refound error was not visible once (couldn't replicate)
 
 
 static int getUnsynchVal(int x)
@@ -725,7 +856,7 @@ struct SortFrm
 
 
 // returns true if all of these happen: pId3V2Stream is ID3V2.3.0, no unsynch is used, the frames are identical except for their order; padding is ignored;
-bool Id3V230StreamWriter::equalTo(Id3V2StreamBase* p) const
+/*bool Id3V230StreamWriter::equalTo(Id3V2StreamBase* p) const
 {
     if (0 == p)
     {
@@ -740,7 +871,7 @@ bool Id3V230StreamWriter::equalTo(Id3V2StreamBase* p) const
     }
 
     return contentEqualTo(p);
-}
+}*/
 
 
 bool Id3V230StreamWriter::contentEqualTo(Id3V2StreamBase* p) const
@@ -770,7 +901,7 @@ bool Id3V230StreamWriter::contentEqualTo(Id3V2StreamBase* p) const
     {
         if (s(v1[i], v2[i]) || s(v2[i], v1[i]))
         {
-            //qDebug("diff content %s %s", v1[i]->m_szName, v2[i]->m_szName);
+            //qDebug("diff content %s (%s) / %s (%s)", v1[i]->m_szName, v1[i]->getUtf8String1().c_str(), v2[i]->m_szName, v2[i]->getUtf8String1().c_str());
             return false;
         }
     }
