@@ -27,6 +27,7 @@
 
 #include  "MpegStream.h"
 #include  "Helpers.h"
+#include  "CommonData.h"
 
 
 using namespace std;
@@ -155,6 +156,7 @@ e1:
 
 Id3V2Frame::Id3V2Frame(streampos pos, bool bHasUnsynch, StringWrp* pFileName) :
         m_nMemDataSize(-1),
+        m_nDiskDataSize(-1),
         m_pos(pos),
         m_pFileName(pFileName),
         m_bHasUnsynch(bHasUnsynch),
@@ -197,12 +199,52 @@ string Id3V2Frame::getReadableName() const
 }
 
 
+static const char* findAfter(const char* p, unsigned char cEnc, const char* pLast)
+{
+//inspect(p, pLast - p);
+    if (0 == p) { return 0; }
+    switch (cEnc)
+    {
+    case 0:
+    case 3:
+        {
+            while (p < pLast - 1 && 0 != *p) { ++p; }
+            if (p + 1 < pLast) { return p + 1; }
+            return 0;
+        }
+
+    case 1:
+    case 2:
+        {
+            while (p < pLast - 2 && (0 != *p || 0 != *(p + 1)))
+            {
+                p += 2;
+                //inspect(p, pLast - p);
+            }
+            if (p + 2 < pLast) { return p + 2; }
+            return 0;
+        }
+
+    default:
+        CB_ASSERT(false);
+    }
+}
+
+
+
 void Id3V2Frame::print(ostream& out, bool bFullInfo) const
 {
     out << m_szName;
     if ('T' == m_szName[0])
     {
-        out << "=\"" << getUtf8String() << "\""; //ttt2 probably specific to particular versions of Linux and GCC
+        string s (getUtf8String());
+        if (!bFullInfo && cSize(s) > 100)
+        {
+            s.erase(90);
+            s += " ...";
+        }
+
+        out << "=\"" << s << "\""; //ttt2 probably specific to particular versions of Linux and GCC
 //cout << " value=\"" << getUtf8String() << "\""; //ttt2 probably specific to particular versions of Linux and GCC
         //out << " value=\"" << "RRRRRRRR" << "\"";
     }
@@ -213,8 +255,20 @@ void Id3V2Frame::print(ostream& out, bool bFullInfo) const
         //const char* q (pData + 1);
         out << ": ";
         int nBeg (1);
-        for (; nBeg < m_nMemDataSize && 0 != pData[nBeg]; ++nBeg) {}
-        ++nBeg;
+
+        { // skip description
+            if (0 == pData[0] || 3 == pData[0])
+            {
+                for (; nBeg < m_nMemDataSize && 0 != pData[nBeg]; ++nBeg) {}
+                ++nBeg;
+            }
+            else
+            {
+                for (; nBeg < m_nMemDataSize - 1 && (0 != pData[nBeg] || 0 != pData[nBeg + 1]); ++nBeg) {}
+                nBeg += 2;
+            }
+        }
+
         QString qs;
         switch (pData[0])
         {
@@ -223,7 +277,14 @@ void Id3V2Frame::print(ostream& out, bool bFullInfo) const
             break;
 
         case 1:
-            qs = QString::fromUtf8(utf8FromBomUtf16(pData + nBeg, m_nMemDataSize - nBeg).c_str());
+            try
+            {
+                qs = QString::fromUtf8(utf8FromBomUtf16(pData + nBeg, m_nMemDataSize - nBeg).c_str());
+            }
+            catch (const NotId3V2Frame&)
+            {
+                qs = "<< error decoding string >>";
+            }
             break;
 
         case 2:
@@ -238,7 +299,7 @@ void Id3V2Frame::print(ostream& out, bool bFullInfo) const
         //qs.replace('\n', " / "); qs.replace('\r', "");
         qs = "\n" + qs + "\n";
 
-        out << qs.toUtf8().data();
+        out << qs.toUtf8().constData();
     }
     else
     {
@@ -270,24 +331,13 @@ void Id3V2Frame::print(ostream& out, bool bFullInfo) const
         else
         {
             const char* pLast (pData + m_nMemDataSize); // actually first after last
-            int nTermSize (1 == cEnc || 2 == cEnc ? 2 : 1);
 
-            pFile = pMime = pData + 1;
-            for (; pFile < pLast && 0 != *pFile; ++pFile) {}
-            if (pFile == pLast) { goto e1; }
-            pFile += 1; // !!! mime is always UTF-8
+            pMime = pData + 1;
 
-            pDescr = pFile;
-            for (; pDescr < pLast && 0 != *pDescr; ++pDescr) {}
-            if (pDescr == pLast) { goto e1; }
-            pDescr += nTermSize;
+            pFile = findAfter(pMime, 0, pLast); // !!! mime is always UTF-8
+            pDescr = findAfter(pFile, cEnc, pLast);
+            pBinData = findAfter(pDescr, cEnc, pLast);
 
-            pBinData = pDescr;
-            for (; pBinData < pLast && 0 != *pBinData; ++pBinData) {}
-            if (pBinData == pLast) { pBinData = 0; goto e1; }
-            pBinData += nTermSize;
-
-e1:
             if (0 != pBinData)
             {
                 qstrMime = QString::fromLatin1(pMime);
@@ -299,8 +349,23 @@ e1:
                     break;
 
                 case 1:
-                    qstrFile = QString::fromUtf8(utf8FromBomUtf16(pFile, pDescr - pFile).c_str());
-                    qstrDescr = QString::fromUtf8(utf8FromBomUtf16(pDescr, pBinData - pDescr).c_str());
+                    try
+                    {
+                        qstrFile = QString::fromUtf8(utf8FromBomUtf16(pFile, pDescr - pFile).c_str());
+                    }
+                    catch (const NotId3V2Frame&)
+                    { // invalid encoding
+                        qstrFile = "<encoding error>";
+                    }
+
+                    try
+                    {
+                        qstrDescr = QString::fromUtf8(utf8FromBomUtf16(pDescr, pBinData - pDescr).c_str());
+                    }
+                    catch (const NotId3V2Frame&)
+                    { // invalid encoding
+                        qstrDescr = "<encoding error>";
+                    }
                     break;
 
                 /*case 2:
@@ -330,27 +395,58 @@ e1:
     }
 
 
-    if (0 == strcmp(KnownFrames::LBL_IMAGE(), m_szName))
+    if (Id3V2Frame::NO_APIC != m_eApicStatus)
     {
-        out << " status=";
-        switch(m_eApicStatus)
-        {
-        case USES_LINK: out << "link"; break;
-        case NOT_SUPPORTED: out << "not supported"; break;
-        case ERROR: out << "error"; break;
-        case OK: out << "OK"; break;
-        default: CB_ASSERT1 (false, m_pFileName->s);
-        }
+        out << " status=" << getImageStatus();
     }
 
 }
 
 
+const char* Id3V2Frame::getImageType() const
+{
+    return ImageInfo::getImageType(m_nPictureType);
+}
+
+
+const char* Id3V2Frame::getImageStatus() const
+{
+    switch(m_eApicStatus)
+    {
+    case USES_LINK: return "link";
+    case NON_COVER: return "non-cover";
+    case ERR: return "error";
+    case COVER: return "cover";
+    default: CB_ASSERT1 (false, m_pFileName->s);
+    }
+}
+
+
+double Id3V2Frame::getRating() const // asserts it's POPM
+{
+    CB_ASSERT (0 == strcmp(KnownFrames::LBL_RATING(), m_szName));
+
+    Id3V2FrameDataLoader wrp (*this);
+    const char* pData (wrp.getData());
+    int k (0);
+    while (k < m_nMemDataSize && 0 != pData[k]) { ++k; } // skip email addr
+    ++k;
+    if (k >= m_nMemDataSize)
+    { // error //ttt2 add warning on constructor
+        return -1;
+    }
+    unsigned char c (pData[k]);
+    return 5*(double(c) - 1)/254;  // ttt2 not sure this is the best mapping
+}
+
+
+
 /*static*/ string Id3V2Frame::utf8FromBomUtf16(const char* pData, int nSize)
 {
     CB_CHECK1 (nSize > 1, NotId3V2Frame()); // UNICODE string entries must have a size of 3 or more."
+    if (2 == nSize && 0 == *pData && 0 == *(pData + 1)) { return ""; } // not quite correct, but seems to happen; even if a string is null, it must begin with BOM //ttt1 add a note
     const unsigned char* p (reinterpret_cast<const unsigned char*> (pData));
-    CB_CHECK1 ((0xff == p[0] && 0xfe == p[1]) || (0xff == p[1] && 0xfe == p[0]), NotId3V2Frame());
+    CB_CHECK1 ((0xff == p[0] && 0xfe == p[1]) || (0xff == p[1] && 0xfe == p[0]), NotId3V2Frame()); //ttt2 perhaps use other exception
 
 #if Q_BYTE_ORDER == Q_LITTLE_ENDIAN
     bool bIsFffeOk (true); // x86
@@ -377,12 +473,90 @@ e1:
     QString qs (QString::fromUtf16(pUs, nUSize));
     string s (convStr(qs));
 
-    rtrim(s);
     return s;
 }
 
 
 
+// for display / export; for frames in KnownFrames::getKnownFrames only the data up to the first 0 is used (effectively removing comments in 2.3.0), while for the others, including TXXX, nulls and all other characters with codes below 32 are replaced with spaces (as a result, both description and value are shown for TXXX), so in either case the return value doesn't contain nulls;
+// whitespaces at the end of the string are removed;
+string Id3V2Frame::getUtf8String() const
+{
+    try
+    {
+        string s (getUtf8StringImpl());
+        if (0 == KnownFrames::getKnownFrames().count(m_szName) || isTxxx())
+        { // text frames may contain null characters, and what's after a null char isn't supposed to be displayed; however, for frames that aren't used we may want to see what's after the null
+            // ttt2 null characters mean different things inside ID3V2.3.0 (comment follows) vs. ID3V2.4.0 (multiple values separated by 0); however, this doesn't matter in current implementation, because Id3V240Frame::getUtf8StringImpl() replaces nulls with commas
+            for (int i = 0; i < cSize(s); ++i)
+            {
+                unsigned char c (s[i]);
+                if (c < 32) // ttt3 ASCII only
+                {
+                    s[i] = 32;
+                }
+            }
+        }
+
+        s = s.c_str(); // !!! so now s doesn't contain null chars
+        rtrim(s);
+
+        return s;
+    }
+    catch (const Id3V2FrameDataLoader::LoadFailure&)
+    {
+        return "<error loading frame>"; //ttt2 not sure if this is the best thing to do, but at least avoids crashes;
+    }
+    catch (const Id3V2Frame::NotId3V2Frame&)
+    {
+        return "<error decoding frame>"; //ttt2 not sure if this is the best thing to do, but at least avoids crashes;
+    }
+}
+
+
+// for internal processing; similar to getUtf8String() but doesn't replace internal null characters with spaces;
+// removes trailing nulls and whitespaces, as well as 2.3.0 comments;
+string Id3V2Frame::getRawUtf8String() const
+{
+    try
+    {
+        string s (getUtf8StringImpl());
+
+        int i (cSize(s) - 1);
+        for (; i >= 0; --i)
+        {
+            unsigned char c (s[i]);
+            if (0 != c && (c >= ' ' || (c < ' ' && QChar(c).isSpace())))
+            {
+                break;
+            }
+        }
+
+        ++i;
+        if (i < cSize(s))
+        {
+            s.erase(i);
+        }
+
+        return s;
+    }
+    catch (const Id3V2FrameDataLoader::LoadFailure&)
+    {
+        return "<error loading frame>"; //ttt2 not sure if this is the best thing to do, but at least avoids crashes;
+    }
+    catch (const Id3V2Frame::NotId3V2Frame&)
+    {
+        return "<error decoding frame>"; //ttt2 not sure if this is the best thing to do, but at least avoids crashes;
+    }
+}
+
+
+bool Id3V2Frame::isTxxx() const
+{
+    return 0 == strcmp(KnownFrames::LBL_TXXX(), m_szName);
+}
+
+
 
 //============================================================================================================
 //============================================================================================================
@@ -393,11 +567,12 @@ Id3V2FrameDataLoader::Id3V2FrameDataLoader(const Id3V2Frame& frame) : m_frame(fr
 {
     if (cSize(frame.m_vcData) < m_frame.m_nMemDataSize)
     {
-        m_bOwnsData = true;
+        int nDiscard (frame.getOffset()); // for DataLengthIndicator
+        //m_bOwnsData = true;
         CB_ASSERT1 (frame.m_vcData.empty(), m_frame.m_pFileName->s);
         CB_ASSERT1 (0 != frame.m_pFileName, m_frame.m_pFileName->s);
-        char* pData (new char[m_frame.m_nMemDataSize]);
-        m_pData = pData;
+
+        m_vcOwnData.resize(m_frame.m_nMemDataSize);
         ifstream_utf8 in (m_frame.m_pFileName->s.c_str(), ios::binary);
         in.seekg(m_frame.m_pos);
         in.seekg(m_frame.m_nDiskHdrSize, ios_base::cur);
@@ -405,28 +580,33 @@ Id3V2FrameDataLoader::Id3V2FrameDataLoader(const Id3V2Frame& frame) : m_frame(fr
         posNext += m_frame.m_nDiskDataSize + m_frame.m_nDiskHdrSize;
         int nContentBytesSkipped (0);
         int nRead (0);
-        nRead = readID3V2(m_frame.m_bHasUnsynch, in, pData, m_frame.m_nMemDataSize, posNext, nContentBytesSkipped);
+        in.seekg(nDiscard, ios_base::cur);
+        nRead = readID3V2(m_frame.m_bHasUnsynch, in, &m_vcOwnData[0], cSize(m_vcOwnData), posNext, nContentBytesSkipped);
         //qDebug("nRead %d ; m_frame.m_nMemDataSize %d ; nContentBytesSkipped %d ", nRead, m_frame.m_nMemDataSize, nContentBytesSkipped);
-        if (m_frame.m_nMemDataSize != nRead)
+        if (cSize(m_vcOwnData) != nRead)
         {
-            delete[] m_pData;
             throw LoadFailure();
         }
+
+        m_pData = &m_vcOwnData[0];
     }
     else
     {
-        m_bOwnsData = false;
-        m_pData = &frame.m_vcData[0];
+        if (frame.m_vcData.empty())
+        {
+            static char c (0);
+            m_pData = &c;
+        }
+        else
+        {
+            m_pData = &frame.m_vcData[0];
+        }
     }
 }
 
 
 Id3V2FrameDataLoader::~Id3V2FrameDataLoader()
 {
-    if (m_bOwnsData)
-    {
-        delete[] m_pData;
-    }
 }
 
 
@@ -452,6 +632,10 @@ Id3V2FrameDataLoader::~Id3V2FrameDataLoader()
 /*static*/ const char* KnownFrames::LBL_RATING() { return "POPM"; }
 /*static*/ const char* KnownFrames::LBL_COMPOSER() { return "TCOM"; }
 
+/*static*/ const char* KnownFrames::LBL_WMP_VAR_ART() { return "TPE2"; }
+/*static*/ const char* KnownFrames::LBL_ITUNES_VAR_ART() { return "TCMP"; }
+
+/*static*/ const char* KnownFrames::LBL_TXXX() { return "TXXX"; }
 
 
 //============================================================================================================
@@ -502,7 +686,7 @@ void Id3V2StreamBase::printFrames(ostream& out) const
 
 /*override*/ void Id3V2StreamBase::copy(std::istream& in, std::ostream& out)
 {
-    appendFilePart(in, out, m_pos, m_nTotalSize); //ttt1
+    appendFilePart(in, out, m_pos, m_nTotalSize); //ttt2
 }
 
 
@@ -667,13 +851,13 @@ void Id3V2StreamBase::checkFrames(NoteColl& notes) // various checks to be calle
         }
         else */if (s.empty())
         {
-            MP3_NOTE (p->m_pos, id3v2EmptyTcon);
+            MP3_NOTE (p->m_pos, id3v2EmptyTcon); //ttt2 perhaps replace this with something more generic
         }
     }
 
     //ttt2 add other checks
 }
-//ttt1 perhaps use links to pictures in crt dir
+//ttt2 perhaps use links to pictures in crt dir
 
 
 /*override*/ ImageInfo Id3V2StreamBase::getImage(bool* pbFrameExists /*= 0*/) const
@@ -736,27 +920,101 @@ e1:
 }
 
 
-
-/*static*/ const char* Id3V2StreamBase::decodeApic(NoteColl& notes, streampos pos, const char* pData, const char*& szMimeType, int& nPictureType, const char*& szDescription)
+/*override*/ vector<ImageInfo> Id3V2StreamBase::getImages() const
 {
-    MP3_CHECK (0 == pData[0] || 3 == pData[0], pos, id3v2UnsupApicTextEnc, NotSupTextEnc()); // !!! there's no need for StreamIsUnsupported here, because this error is not fatal, and it isn't allowed to propagate, therefore doesn't cause a stream to be Unsupported; //ttt1 review, support
-    ++pData;
-    szMimeType = pData; // ttt3 type 0 is Latin1, while type 3 is UTF8, so this isn't quite right; however, MIME types should probably be plain ASCII, so it's the same; and anyway, we only recognize JPEG and PNG, which are ASCII
-    int nMimeSize (strlen(pData)); //ttt1 doesn't work for corrupted pData, when there is no 0 terminator
+    vector<ImageInfo> v;
 
-    pData += 1 + nMimeSize;
-    nPictureType = *pData++;
+    for (int i = 0; i < cSize(m_vpFrames); ++i)
+    {
+        Id3V2Frame* pFrame (m_vpFrames[i]);
+        if (Id3V2Frame::NON_COVER == pFrame->m_eApicStatus || Id3V2Frame::COVER == pFrame->m_eApicStatus)
+        {
+            try
+            {
+                Id3V2FrameDataLoader wrp (*pFrame);
+                const char* pCrtData (wrp.getData());
+                const char* pBinData (pCrtData + pFrame->m_nImgOffset);
+                //CB_CHECK (pixmap.loadFromData(pBinData, m_nImgSize));
 
-    szDescription = pData;
+                // make sure the data is still available and correct (the file might have been modified externally)
+                if (-1 == pFrame->m_nWidth)
+                {
+                    QPixmap pic;
+                    if (!pic.loadFromData(reinterpret_cast<const unsigned char*>(pBinData), pFrame->m_nImgSize)) // this takes a lot of time
+                    {
+                        v.push_back(ImageInfo(pFrame->m_nPictureType, ImageInfo::ERROR_LOADING));
+                        continue;
+                    }
+                    pFrame->m_nWidth = short(pic.width());
+                    pFrame->m_nHeight = short(pic.height());
+                }
 
-    return pData + strlen(szDescription) + 1;
+                QByteArray b (QByteArray::fromRawData(pBinData, pFrame->m_nImgSize));
+                b.append('x'); b.resize(b.size() - 1); // !!! these are needed because fromRawData() doesn't create copies of the memory used for the byte array
+                v.push_back(ImageInfo(pFrame->m_nPictureType, Id3V2Frame::COVER == pFrame->m_eApicStatus ? ImageInfo::OK : ImageInfo::LOADED_NOT_COVER, pFrame->m_eCompr, b, pFrame->m_nWidth, pFrame->m_nHeight));
+            }
+            catch (const Id3V2FrameDataLoader::LoadFailure&)
+            {
+                v.push_back(ImageInfo(pFrame->m_nPictureType, ImageInfo::ERROR_LOADING));
+            }
+
+            continue;
+        }
+
+        if (Id3V2Frame::ERR == pFrame->m_eApicStatus)
+        {
+            v.push_back(ImageInfo(pFrame->m_nPictureType, ImageInfo::ERROR_LOADING));
+        }
+
+        if (Id3V2Frame::USES_LINK == pFrame->m_eApicStatus)
+        {
+            v.push_back(ImageInfo(pFrame->m_nPictureType, ImageInfo::USES_LINK));
+        }
+    }
+
+    return v;
 }
 
 
-static bool isTypeSupported(int nType)
+/*override*/ std::string Id3V2StreamBase::getImageData(bool* pbFrameExists /*= 0*/) const
 {
-    //return Id3V2Frame::OTHER == nType || Id3V2Frame::ICON == nType || Id3V2Frame::COVER == nType; //ttt1 review decision to have all these map to "cover"; see also Mp3HandlerTagData::reload(), where saving of a "cover" image is influenced by what this returned
-    return Id3V2Frame::COVER == nType; // 2009.04.05 - for a while it seemed a good idea to report OTHER and ICON as "supported", but there's the issue of what to do when deleting an image in the tag editor; seems better to just use the cover; //ttt1 OTOH this creates more duplicates
+    if (0 != pbFrameExists)
+    {
+        *pbFrameExists = 0 != m_pPicFrame;
+    }
+
+    if (0 == m_pPicFrame) { return ""; }
+
+    ostringstream out;
+    out << "type:" << m_pPicFrame->getImageType() << ", status:" << m_pPicFrame->getImageStatus() << ", size:" << m_pPicFrame->m_nWidth << "x" << m_pPicFrame->m_nHeight << ", compr:" << ImageInfo::getComprStr(m_pPicFrame->m_eCompr);
+
+    return out.str();
+}
+
+/*static*/ const char* Id3V2StreamBase::decodeApic(NoteColl& notes, int nDataSize, streampos pos, const char* const pData, const char*& szMimeType, int& nPictureType, const char*& szDescription)
+{
+    MP3_CHECK (0 == pData[0] || 3 == pData[0], pos, id3v2UnsupApicTextEnc, NotSupTextEnc()); // !!! there's no need for StreamIsUnsupported here, because this error is not fatal, and it isn't allowed to propagate, therefore doesn't cause a stream to be Unsupported; //ttt2 review, support
+    szMimeType = pData + 1; // ttt3 type 0 is Latin1, while type 3 is UTF8, so this isn't quite right; however, MIME types should probably be plain ASCII, so it's the same; and anyway, we only recognize JPEG and PNG, which are ASCII
+    //int nMimeSize (strnlen(pData, nDataSize));
+
+    const char* p (pData + 1);
+    for (; p < pData + nDataSize && 0 != *p; ++p) {}
+    MP3_CHECK (p < pData + nDataSize - 1, pos, id3v2UnsupApicTextEnc, ErrorDecodingApic()); // "-1" to account for szDescription
+    CB_ASSERT (0 == *p);
+    ++p;
+
+    nPictureType = *p++;
+
+    szDescription = p;
+    for (; p < pData + nDataSize; ++p)
+    {
+        if (0 == *p)
+        {
+            return p + 1;
+        }
+    }
+
+    throw ErrorDecodingApic();
 }
 
 
@@ -778,7 +1036,7 @@ void Id3V2StreamBase::preparePictureHlp(NoteColl& notes, Id3V2Frame* pFrame, con
     {
         pFrame->m_nImgSize = nSize;
         pFrame->m_nImgOffset = pImgData - pFrameData;
-        pFrame->m_eApicStatus = Id3V2Frame::OK;
+        pFrame->m_eApicStatus = pFrame->m_nPictureType == Id3V2Frame::PT_COVER ? Id3V2Frame::COVER : Id3V2Frame::NON_COVER;
         pFrame->m_nWidth = short(img.width());
         pFrame->m_nHeight = short(img.height());
         if (0 == strcmp("image/jpeg", szMimeType) || 0 == strcmp("image/jpg", szMimeType))
@@ -792,11 +1050,11 @@ void Id3V2StreamBase::preparePictureHlp(NoteColl& notes, Id3V2Frame* pFrame, con
         else
         {
             pFrame->m_eCompr = ImageInfo::INVALID;
-        } //ttt1 perhaps support GIF or other formats
+        } //ttt2 perhaps support GIF or other formats; (well, GIFs can be loaded, but are recompressed when saving)
         return;
     }
 
-    pFrame->m_eApicStatus = Id3V2Frame::ERROR;
+    pFrame->m_eApicStatus = Id3V2Frame::ERR;
 
     if (pFrame->m_nMemDataSize > 100)
     {
@@ -814,15 +1072,16 @@ void Id3V2StreamBase::preparePicture(NoteColl& notes) // initializes fields used
 {
     const char* szMimeType;
     const char* szDescription;
-    Id3V2Frame* pFirstValidApicFrame (0);
-    Id3V2Frame* pFirstApicFrame (0);
+    //Id3V2Frame* pFirstLoadableCover (0);
+    Id3V2Frame* pFirstLoadableNonCover (0);
+    Id3V2Frame* pFirstApic (0); // might have errors, might be link, ...
 
     for (int i = 0, n = cSize(m_vpFrames); i < n; ++i)
-    {
+    { // go through the frame list and set m_eApicStatus
         Id3V2Frame* p = m_vpFrames[i];
         if (0 == strcmp(KnownFrames::LBL_IMAGE(), p->m_szName))
         {
-            if (0 == pFirstApicFrame) { pFirstApicFrame = p; }
+            if (0 == pFirstApic) { pFirstApic = p; } // !!! regardless of what exceptions might get thrown, p will have an m_eApicStatus other than NO_APIC; that happens either in preparePictureHlp() or with explicit assignments
             try
             {
                 Id3V2FrameDataLoader wrp (*p);
@@ -831,34 +1090,41 @@ void Id3V2StreamBase::preparePicture(NoteColl& notes) // initializes fields used
 
                 try
                 {
-                    pCrtData = decodeApic(notes, p->m_pos, pData, szMimeType, p->m_nPictureType, szDescription);
+                    pCrtData = decodeApic(notes, p->m_nMemDataSize, p->m_pos, pData, szMimeType, p->m_nPictureType, szDescription);
                 }
                 catch (const NotSupTextEnc&)
                 {
-                    p->m_eApicStatus = Id3V2Frame::NOT_SUPPORTED;
+                    p->m_eApicStatus = Id3V2Frame::ERR;
                     continue;
                 }
+                catch (const ErrorDecodingApic&)
+                {
+                    p->m_eApicStatus = Id3V2Frame::ERR;
+                    continue;
+                }
+
                 if (0 != *szDescription) { MP3_NOTE (p->m_pos, id3v2PictDescrIgnored); }
 
                 preparePictureHlp(notes, p, pData, pCrtData, szMimeType);
 
-                if (Id3V2Frame::OK == p->m_eApicStatus)
+                if (Id3V2Frame::COVER == p->m_eApicStatus)
                 {
-                    if (isTypeSupported(p->m_nPictureType))
+                    if (0 == m_pPicFrame)
                     {
                         m_eImageStatus = ImageInfo::OK;
                         m_pPicFrame = p;
                     }
-                    else if (0 == pFirstValidApicFrame)
-                    {
-                        pFirstValidApicFrame = p;
-                    }
                 }
 
+                if (0 == pFirstLoadableNonCover && Id3V2Frame::NON_COVER == p->m_eApicStatus)
+                {
+                    pFirstLoadableNonCover = p;
+                }
             }
             catch (const Id3V2FrameDataLoader::LoadFailure&)
             {
                 MP3_NOTE (p->m_pos, fileWasChanged);
+                p->m_eApicStatus = Id3V2Frame::ERR;
             }
         }
     }
@@ -869,25 +1135,25 @@ void Id3V2StreamBase::preparePicture(NoteColl& notes) // initializes fields used
         return;
     }
 
-    // no frame with supported type (0, 1, or 3) was found; just pick the first APIC frame and use it
+    // no cover frame was found; just pick the first loadable APIC frame and use it
+    CB_ASSERT (ImageInfo::NO_PICTURE_FOUND == m_eImageStatus);
 
-    if (ImageInfo::NO_PICTURE_FOUND == m_eImageStatus && 0 != pFirstValidApicFrame)
+    if (0 != pFirstLoadableNonCover)
     {
         m_eImageStatus = ImageInfo::LOADED_NOT_COVER;
-        m_pPicFrame = pFirstValidApicFrame;
+        m_pPicFrame = pFirstLoadableNonCover;
         return;
     }
 
-    if (0 == pFirstApicFrame)
+    if (0 == pFirstApic)
     {
         return;
     }
 
-    switch (pFirstApicFrame->m_eApicStatus)
+    switch (pFirstApic->m_eApicStatus)
     {
     case Id3V2Frame::USES_LINK: m_eImageStatus = ImageInfo::USES_LINK; return;
-    case Id3V2Frame::ERROR: m_eImageStatus = ImageInfo::ERROR_LOADING; return;
-    case Id3V2Frame::NOT_SUPPORTED: m_eImageStatus = ImageInfo::ERROR_LOADING; return;
+    case Id3V2Frame::ERR: m_eImageStatus = ImageInfo::ERROR_LOADING; return;
     default: CB_ASSERT1 (false, m_pFileName->s); // all cases should have been covered
     }
 
@@ -914,23 +1180,86 @@ void Id3V2StreamBase::preparePicture(NoteColl& notes) // initializes fields used
 }
 
 
+// *pbFrameExists gets set if at least one frame exists
+/*override*/ int Id3V2StreamBase::getVariousArtists(bool* pbFrameExists /*= 0*/) const
+{
+    int nRes (0);
+    if (0 != pbFrameExists) { *pbFrameExists = false; }
+
+    try
+    {
+        {
+            const Id3V2Frame* p (findFrame(KnownFrames::LBL_WMP_VAR_ART()));
+            if (0 != p)
+            {
+                if (0 != pbFrameExists)
+                {
+                    *pbFrameExists = true;
+                }
+                if (0 == convStr(p->getUtf8String()).compare("VaRiOuS Artists", Qt::CaseInsensitive))
+                {
+                    nRes += TagReader::VA_WMP;
+                }
+            }
+        }
+
+        {
+            const Id3V2Frame* p (findFrame(KnownFrames::LBL_ITUNES_VAR_ART()));
+            if (0 != p)
+            {
+                if (0 != pbFrameExists)
+                {
+                    *pbFrameExists = true;
+                }
+                if ("1" == p->getUtf8String())
+                {
+                    nRes += TagReader::VA_ITUNES;
+                }
+            }
+        }
+    }
+    catch (const Id3V2Frame::NotId3V2Frame&)
+    { // !!! nothing
+    }
+    catch (const Id3V2Frame::UnsupportedId3V2Frame&)
+    { // !!! nothing
+    }
+
+    return nRes;
+}
+
+
 /*override*/ double Id3V2StreamBase::getRating(bool* pbFrameExists /*= 0*/) const
 {
     const Id3V2Frame* p (findFrame(KnownFrames::LBL_RATING()));
     if (0 != pbFrameExists) { *pbFrameExists = 0 != p; }
     if (0 == p) { return -1; }
-    Id3V2FrameDataLoader wrp (*p);
-    const char* pData (wrp.getData());
-    int n (p->m_nMemDataSize);
-    int k (0);
-    while (k < n && 0 != pData[k]) { ++k; } // skip email addr
-    ++k;
-    if (k >= n)
-    { // error //ttt2 add warning on constructor
-        return -1;
+
+    return p->getRating();
+}
+
+
+static const char* REPLAY_GAIN_ROOT ("replaygain");
+
+
+//ttt1 mp3gain 1.5.1 can use ID3V2 instead of APE, so that should be tested as well; see TXXX="MP3GAIN_MINMAX[...]" in "Arctic Monkeys - 01 - My Propeller.mp3" (which also has TXXX="replaygain_track_gain[...]")
+bool Id3V2StreamBase::hasReplayGain() const
+{
+    for (int i = 0; i < cSize(m_vpFrames); ++i)
+    {
+        const Id3V2Frame* pFrame (m_vpFrames[i]);
+        if (pFrame->isTxxx())
+        {
+            QString qs (convStr(pFrame->getUtf8String()));
+
+            if (qs.startsWith(REPLAY_GAIN_ROOT, Qt::CaseInsensitive))
+            {
+                return true;
+            }
+        }
     }
-    unsigned char c (pData[k]);
-    return 5*(double(c) - 1)/254;  // ttt1 not sure this is the best mapping
+
+    return false;
 }
 
 
@@ -960,23 +1289,39 @@ const Id3V2Frame* Id3V2StreamBase::getFrame(const char* szName) const
 }
 
 
+/*static*/ const set<string>& KnownFrames::getExcludeFromInfoFrames() // frames that shouldn't be part of "other info"; doesn't include TXXX and "Various Artists" frames
+{
+    static bool bFirstTime (true);
+    static set<string> s;
+    if (bFirstTime)
+    {
+        s.insert(KnownFrames::LBL_TITLE());
+        s.insert(KnownFrames::LBL_ARTIST());
+        s.insert(KnownFrames::LBL_TRACK_NUMBER());
+        s.insert(KnownFrames::LBL_TIME_YEAR_230());
+        s.insert(KnownFrames::LBL_TIME_DATE_230());
+        s.insert(KnownFrames::LBL_TIME_240()); //ttt2 perhaps this shouldn't be used for 2.3.0, but it covers cases like reading 2.4.0 and writing 2.3.0 or bugs by some tools
+        s.insert(KnownFrames::LBL_GENRE());
+        s.insert(KnownFrames::LBL_IMAGE());
+        s.insert(KnownFrames::LBL_ALBUM());
+        s.insert(KnownFrames::LBL_RATING());
+        s.insert(KnownFrames::LBL_COMPOSER());
+
+        bFirstTime = false;
+    }
+
+    return s;
+}
+
+// includes "Various Artists" frames; doesn't include TXXX
 /*static*/ const set<string>& KnownFrames::getKnownFrames()
 {
     static bool bFirstTime (true);
-    static set<string> sKnownFrames;
+    static set<string> sKnownFrames (getExcludeFromInfoFrames());
     if (bFirstTime)
     {
-        sKnownFrames.insert(KnownFrames::LBL_TITLE());
-        sKnownFrames.insert(KnownFrames::LBL_ARTIST());
-        sKnownFrames.insert(KnownFrames::LBL_TRACK_NUMBER());
-        sKnownFrames.insert(KnownFrames::LBL_TIME_YEAR_230());
-        sKnownFrames.insert(KnownFrames::LBL_TIME_DATE_230());
-        sKnownFrames.insert(KnownFrames::LBL_TIME_240()); //ttt2 perhaps this shouldn't be used for 2.3.0, but it covers cases like reading 2.4.0 and writing 2.3.0 or bugs by some tools
-        sKnownFrames.insert(KnownFrames::LBL_GENRE());
-        sKnownFrames.insert(KnownFrames::LBL_IMAGE());
-        sKnownFrames.insert(KnownFrames::LBL_ALBUM());
-        sKnownFrames.insert(KnownFrames::LBL_RATING());
-        sKnownFrames.insert(KnownFrames::LBL_COMPOSER());
+        sKnownFrames.insert(KnownFrames::LBL_WMP_VAR_ART());
+        sKnownFrames.insert(KnownFrames::LBL_ITUNES_VAR_ART());
 
         bFirstTime = false;
     }
@@ -988,8 +1333,6 @@ const Id3V2Frame* Id3V2StreamBase::getFrame(const char* szName) const
 
 /*override*/ std::string Id3V2StreamBase::getOtherInfo() const
 {
-    const set<string>& sKnownFrames (KnownFrames::getKnownFrames());
-
     set<string> sUsedFrames;
 
     //string strRes;
@@ -999,15 +1342,18 @@ const Id3V2Frame* Id3V2StreamBase::getFrame(const char* szName) const
     for (int i = 0, n = cSize(m_vpFrames); i < n; ++i)
     {
         Id3V2Frame* p = m_vpFrames[i];
-        if (sKnownFrames.count(p->m_szName) > 0 && sUsedFrames.count(p->m_szName) == 0)
+        if (KnownFrames::getExcludeFromInfoFrames().count(p->m_szName) > 0 && sUsedFrames.count(p->m_szName) == 0)
         {
             sUsedFrames.insert(p->m_szName);
         }
         else
         {
-            if (b) { out << ", "; }
-            b = true;
-            p->print(out, Id3V2Frame::FULL_INFO);
+            if (Id3V2Frame::NON_COVER != p->m_eApicStatus && Id3V2Frame::COVER != p->m_eApicStatus) // images that can be loaded are shown, so there shouldn't be another entry for them
+            {
+                if (b) { out << ", "; }
+                b = true;
+                p->print(out, Id3V2Frame::FULL_INFO);
+            }
         }
     }
     return out.str();
@@ -1018,7 +1364,7 @@ const Id3V2Frame* Id3V2StreamBase::getFrame(const char* szName) const
 void Id3V2StreamBase::checkDuplicates(NoteColl& notes) const
 {
     // for some it's OK to be duplicated, e.g. for APIC and various "Picture type" pictures;
-    const set<string>& sKnownFrames (KnownFrames::getKnownFrames());
+
     set<pair<string, int> > sUsedFrames;
     int nImgCnt (0);
     streampos secondImgPos (-1);
@@ -1029,7 +1375,7 @@ void Id3V2StreamBase::checkDuplicates(NoteColl& notes) const
 //if (0 == strcmp("TDOR", p->m_szName)) { MP3_NOTE (p->m_pos, "TDOR found. See if it should be processed."); } //ttt remove
 //if (0 == strcmp("TDRC", p->m_szName)) { MP3_NOTE (p->m_pos, "TDRC found. See if it should be processed."); }
 //if (0 == strcmp("TDRL", p->m_szName)) { MP3_NOTE (p->m_pos, "TDRL found. See if it should be processed."); }
-        if (sKnownFrames.count(p->m_szName) > 0)
+        if (KnownFrames::getKnownFrames().count(p->m_szName) > 0)
         {
             if (0 == strcmp(KnownFrames::LBL_IMAGE(), p->m_szName))
             {
@@ -1039,6 +1385,7 @@ void Id3V2StreamBase::checkDuplicates(NoteColl& notes) const
                     secondImgPos = p->m_pos;
                 }
             }
+
             if (sUsedFrames.count(make_pair(p->m_szName, p->m_nPictureType)) == 0)
             {
                 sUsedFrames.insert(make_pair(p->m_szName, p->m_nPictureType));
@@ -1100,18 +1447,38 @@ vector<const Id3V2Frame*> Id3V2StreamBase::getKnownFrames() const // to be used
     {
         const Id3V2Frame* p (m_vpFrames[i]);
 
-        if (KnownFrames::getKnownFrames().count(p->m_szName) > 0)
+        if (KnownFrames::getKnownFrames().count(p->m_szName) > 0 || p->isTxxx())
         {
+            // add if known except if a picture with the same type was already added; don't add duplicates even if Id3V230StreamWriter could take care of them, because it keeps the last value and we want the first one
             bool bAdd (true);
             for (int j = 0; j < cSize(v); ++j)
             {
                 if (p->m_nPictureType == v[j]->m_nPictureType && 0 == strcmp(v[j]->m_szName, p->m_szName))
                 {
+                    // !!! important for both image and non-image frames; while Id3V230StreamWriter would remove duplicates as configured, we want to get rid of them now, so the first value is kept; Id3V230StreamWriter removes old values as new ones are added, which would keep the last value
+                    //ttt2 not always right for multiple pictures if "keep one" is checked;
+
                     bAdd = false;
                     break;
                 }
             }
 
+            if (p->isTxxx())
+            {
+                string s (p->getUtf8String());
+                QString qs (convStr(s));
+
+                if (qs.startsWith(REPLAY_GAIN_ROOT, Qt::CaseInsensitive))
+                {
+                    bAdd = true; //ttt2 perhaps check it's no duplicate; not sure about case
+                }
+            }
+
+            if (bAdd && 0 == strcmp(KnownFrames::LBL_IMAGE(), p->m_szName) && Id3V2Frame::COVER != p->m_eApicStatus && Id3V2Frame::NON_COVER != p->m_eApicStatus)
+            { // !!! get rid of broken pictures and links
+                bAdd = false;
+            }
+
             if (bAdd)
             {
                 v.push_back(p);
@@ -1171,3 +1538,4 @@ vector<const Id3V2Frame*> Id3V2StreamBase::getKnownFrames() const // to be used
     return true;
 }
 
+//ttt2 note for mismatch between file name and rating
