@@ -19,6 +19,12 @@
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 
+
+#ifdef MSVC_QMAKE
+    #pragma warning (disable : 4100)
+#endif
+
+
 #include  <algorithm>
 
 #include  <QFile>
@@ -38,6 +44,7 @@
 #include  "Id3V240Stream.h"
 #include  "Mp3Manip.h"
 #include  "CommonData.h"
+#include  "MpegStream.h"
 
 ////#include  <iostream> //ttt remove
 
@@ -69,6 +76,7 @@ namespace
     }
 }
 
+
 ostream& operator<<(ostream& out, const TagWriter::OrigValue& val)
 {
     out << "song: " << val.m_nSong << ", field: " << val.m_nField << ", status: " << (int)val.m_eStatus << ", value: " << val.m_strVal;
@@ -84,7 +92,7 @@ ImageColl::ImageColl() : m_nCurrent(-1)
 {
 }
 
-int ImageColl::addImage(const ImageInfo& img, const string& strFile /*= ""*/) // returns the index of the image; if it already exists it's not added again; if it's invalid returns -1
+int ImageColl::addImage(const ImageInfo& img, const string& strFile /*= ""*/) // returns the index of the image; if it already exists it's not added again; if it's invalid returns -1 //ttt1 recompress large images AND mark field as changed
 {
     if (img.isNull()) { return -1; }
     int n (find(img));
@@ -228,6 +236,7 @@ TrackTextReader::TrackTextReader(SongInfoParser::TrackTextParser* pTrackTextPars
     case ALBUM:
     case RATING:
     case COMPOSER:
+    //case VARIOUS_ARTISTS:
         return READ_ONLY;
 
     default:
@@ -263,16 +272,19 @@ WebReader::WebReader(const AlbumInfo& albumInfo, int nTrackNo) : m_strType(album
     m_strAlbumName = albumInfo.m_strTitle;
     m_dRating = ti.m_dRating;
     m_strComposer = ti.m_strComposer; //(ti.m_strComposer.empty() ? albumInfo.m_strComposer : ti.m_strComposer);
+    m_eVarArtists = albumInfo.m_eVarArtists;
 
     if (DiscogsDownloader::SOURCE_NAME == albumInfo.m_strSourceName)
     {
         m_bSuppGenre = true;
         m_bSuppComposer = true;
+        m_bSuppVarArtists = false;
     }
     else if (MusicBrainzDownloader::SOURCE_NAME == albumInfo.m_strSourceName)
     {
         m_bSuppGenre = false;
         m_bSuppComposer = false;
+        m_bSuppVarArtists = true;
     }
     else
     {
@@ -307,6 +319,9 @@ WebReader::~WebReader()
     case COMPOSER:
         return m_bSuppComposer ? READ_ONLY : NOT_SUPPORTED;
 
+    case VARIOUS_ARTISTS:
+        return m_bSuppVarArtists ? READ_ONLY : NOT_SUPPORTED;
+
     case RATING:
     default:
         return NOT_SUPPORTED;
@@ -314,6 +329,25 @@ WebReader::~WebReader()
 }
 
 
+int WebReader::convVarArtists() const // converts m_eVarArtists to an int is either 0 or contains all VA-enabled values, based on configuration
+{
+    switch (m_eVarArtists)
+    {
+    case AlbumInfo::VA_NOT_SUPP:
+    case AlbumInfo::VA_SINGLE:
+        return TagReader::VA_NONE;
+    case AlbumInfo::VA_VARIOUS:
+        {
+            const CommonData* p (getCommonData());
+            int nRes (0);
+            if (p->m_bItunesVarArtists) { nRes += TagReader::VA_ITUNES; }
+            if (p->m_bWmpVarArtists) { nRes += TagReader::VA_WMP; }
+            return nRes;
+        }
+    default: CB_ASSERT (false);
+    }
+}
+
 
 //======================================================================================================================
 //======================================================================================================================
@@ -323,7 +357,7 @@ Mp3HandlerTagData::Mp3HandlerTagData(TagWriter* pTagWriter, const Mp3Handler* pM
 {
     TRACER("Mp3HandlerTagData constr");
     refreshReaders();
-    reload();
+    setUp();
     //qDebug("create %p", this);
 }
 
@@ -343,15 +377,54 @@ static bool isId3V2(const TagReader* p)
 static const char* g_szImageFmt ("# %d");
 extern const int PIC_FMT_HDR (2);
 
-// to be called initially and each time the priority of tag readers changes
-void Mp3HandlerTagData::reload()
+
+void Mp3HandlerTagData::adjustVarArtists(bool b) // if VARIOUS_ARTISTS is not ASSIGNED, sets m_strValue and m_eStatus
+{
+    ValueInfo& inf (m_vValueInfo[TagReader::VARIOUS_ARTISTS]);
+    string strAll (TagReader::getVarArtistsValue());
+
+    if (ASSIGNED != inf.m_eStatus)
+    {
+        inf.m_strValue.clear();
+
+        string s (b ? strAll : "");
+        inf.m_strValue = s;
+        inf.m_eStatus = s.empty() ? EMPTY : NON_ID3V2_VAL;
+
+        for (int i = 0, n = cSize(m_vpTagReaders); i < n; ++i)
+        {
+            TagReader* p (m_vpTagReaders[i]);
+
+            if (isId3V2(p))
+            {
+                string strId3 (p->getValue(TagReader::VARIOUS_ARTISTS));
+                string s1; // those letters from strId3 that are also in strAll; needed because strId3 contains all known letters, regardless of their being enabled
+                for (int i = 0; i < cSize(strId3); ++i)
+                {
+                    if (string::npos != strAll.find(strId3[i]))
+                    {
+                        s1 += strId3[i];
+                    }
+                }
+
+                inf.m_eStatus = s1 == s ? ID3V2_VAL : NON_ID3V2_VAL;
+
+                break;
+            }
+        }
+    }
+}
+
+
+// to be called initially and each time the priority of tag readers changes; (well, actually a new object will get constructed in the latter case, so only the first matters)
+void Mp3HandlerTagData::setUp()
 {
 
 //cout << "reload " << this; for (int i = 0, n = cSize(m_vpTagReaders); i < n; ++i) { cout << " " << m_vpTagReaders[i]; } cout << endl; //qDebug("%s", p->getName());
 
     for (int f = 0; f < TagReader::LIST_END; ++f)
     {
-        if (TagReader::IMAGE != f) // special case needed because TagReader::getValue(TagReader::IMAGE) always returns an empty string;
+        if (TagReader::IMAGE != f && TagReader::VARIOUS_ARTISTS != f) // special case needed because TagReader::getValue(TagReader::IMAGE) always returns an empty string; VARIOUS_ARTISTS is handled separately anyway
         {
             ValueInfo& inf (m_vValueInfo[f]);
             if (ASSIGNED != inf.m_eStatus)
@@ -368,26 +441,51 @@ void Mp3HandlerTagData::reload()
                     string s (p->getValue((TagReader::Feature)f));
                     if (!s.empty())
                     {
+                        bool bId3V2Val (true);
+
+                        if ((TagReader::ARTIST == f || TagReader::COMPOSER == f) && TC_NONE != m_pTagWriter->m_eArtistCase)
+                        {
+                            string s1 (convStr(getCaseConv(convStr(s), m_pTagWriter->m_eArtistCase)));
+                            if (s1 != s)
+                            {
+                                s = s1;
+                                bId3V2Val = false;
+                            }
+                        }
+
+                        if ((TagReader::TITLE == f || TagReader::ALBUM == f) && TC_NONE != m_pTagWriter->m_eTitleCase) //ttt2 perhaps include genre as well
+                        {
+                            string s1 (convStr(getCaseConv(convStr(s), m_pTagWriter->m_eTitleCase)));
+                            if (s1 != s)
+                            {
+                                s = s1;
+                                bId3V2Val = false;
+                            }
+                        }
+
                         inf.m_strValue = s;
 
-                        bool bId3V2Val (false);
-                        if (isId3V2(p))
+                        if (bId3V2Val)
                         {
-                            bId3V2Val = true;
-                        }
-                        else if (bId3V2Found)
-                        { // !!! nothing: an ID3V2 was already found and, since it got here, the value was empty
-                        }
-                        else
-                        { // find the first ID3V2 and see if it has the same value
-                            for (; i < n; ++i)
+                            bId3V2Val = false; // !!!
+                            if (isId3V2(p))
                             {
-                                TagReader* p1 (m_vpTagReaders[i]);
-                                if (isId3V2(p1))
+                                bId3V2Val = true;
+                            }
+                            else if (bId3V2Found)
+                            { // !!! nothing: an ID3V2 was already found and, since it got here, the value was empty
+                            }
+                            else
+                            { // find the first ID3V2 and see if it has the same value
+                                for (; i < n; ++i)
                                 {
-                                    string s1 (p1->getValue((TagReader::Feature)f));
-                                    bId3V2Val = (s1 == s);
-                                    break;
+                                    TagReader* p1 (m_vpTagReaders[i]);
+                                    if (isId3V2(p1))
+                                    {
+                                        string s1 (p1->getValue((TagReader::Feature)f));
+                                        bId3V2Val = (s1 == s);
+                                        break;
+                                    }
                                 }
                             }
                         }
@@ -474,6 +572,11 @@ void Mp3HandlerTagData::setData(int nField, const std::string& s)
 {
     if (s == m_vValueInfo[nField].m_strValue) { return; }
 
+    if (TagReader::ARTIST == nField)
+    {
+        m_pTagWriter->delayedAdjVarArtists();
+    }
+
     if (TagReader::TIME == nField)
     {
         try
@@ -529,7 +632,7 @@ void Mp3HandlerTagData::setStatus(int nField, Status eStatus)
 }
 
 
-//ttt0 ??? ptr deallocated and then reallocated in the same place most of the time?
+//ttt2 ??? ptr deallocated and then reallocated in the same place most of the time?
 // returns the data corresponding to the k-th element in m_pTagWriter->m_vTagReaderInfo; returns "\1" if it doesn't have a corresponding stream (e.g. 2nd ID3V1 tag), "\2" if the given feature is not supported (e.g. picture in ID3V1) and "\3" if this particular tag doesn't have the requested frame
 // nField is the "internal" row for which data is retrieved, so TagReader::FEATURE_ON_POS[] has to be used by the UI caller
 std::string Mp3HandlerTagData::getData(int nField, int k) const
@@ -586,11 +689,19 @@ std::string Mp3HandlerTagData::getData(int nField, int k) const
             char a [15];
             double d (p->getRating(&bFrameExists));
             if (!bFrameExists) { return "\3"; }
+            //ttt2 perhaps use this: return p->getValue(TagReader::RATING); see why there's no test for "<0" in TagReader::getValue()
             sprintf(a, "%0.1f", d);
             if ('-' == a[0]) { a[0] = 0; }
             return a;
         }
     case TagReader::COMPOSER: { if (TagReader::NOT_SUPPORTED == p->getSupport(TagReader::COMPOSER)) { return "\2"; } string s (p->getComposer(&bFrameExists)); return bFrameExists ? s : "\3"; }
+    case TagReader::VARIOUS_ARTISTS:
+        {
+            if (TagReader::NOT_SUPPORTED == p->getSupport(TagReader::VARIOUS_ARTISTS)) { return "\2"; }
+            p->getVariousArtists(&bFrameExists);
+            if (!bFrameExists) { return "\3"; }
+            return p->getValue(TagReader::VARIOUS_ARTISTS);
+        }
     }
 
     CB_ASSERT(false); // all cases have been covered and have "return"
@@ -607,7 +718,7 @@ int Mp3HandlerTagData::getImage() const // 0-based; -1 if there-s no image;
 double Mp3HandlerTagData::getRating() const
 {
     string s (getData(TagReader::RATING));
-    double d (s.empty() ? -1 : atof(s.c_str())); //ttt1 review atof/sprintf usage, for foreign locales;
+    double d (s.empty() ? -1 : atof(s.c_str())); // !!! it's OK to use atof, regardless of the system's locale, because a program's locale is "C" unless changed explicitley ("locale::global(locale(""));" would switch to the system's locale)
     return d;
 }
 
@@ -706,7 +817,7 @@ const TagReader* Mp3HandlerTagData::getMatchingReader(int i) const
 
 
 
-TagWriter::TagWriter(CommonData* pCommonData, QWidget* pParentWnd, const bool& bIsFastSaving) : m_pCommonData(pCommonData), m_pParentWnd(pParentWnd), m_nCurrentFile(-1), m_bShowedNonSeqWarn(true), m_bIsFastSaving(bIsFastSaving), m_nFileToErase(-1)
+TagWriter::TagWriter(CommonData* pCommonData, QWidget* pParentWnd, const bool& bIsFastSaving, const TextCaseOptions& eArtistCase, const TextCaseOptions& eTitleCase) : m_pCommonData(pCommonData), m_pParentWnd(pParentWnd), m_nCurrentFile(-1), m_bShowedNonSeqWarn(true), m_bIsFastSaving(bIsFastSaving), m_nFileToErase(-1), m_bVariousArtists(false), m_bAutoVarArtists(false), m_bDelayedAdjVarArtists(false), m_bWaitingChangeNotif(false), m_eArtistCase(eArtistCase),  m_eTitleCase(eTitleCase)
 {
 }
 
@@ -792,12 +903,20 @@ namespace
 {
     struct SortByTrack
     {
-        static int getTrack(const string& s) // returns -1 if it can't identify a track number
+        enum { NO_TRACK = 999999 };
+        static double getTrack(const string& s) // returns -1 if it can't identify a track number
         {
             const char* p (s.c_str()); // requiring all chars to be digits isn't OK, because "5/12" should be handled as "5"
             for (; ' ' == *p; ++p) {}
-            if (0 == *p || !isdigit(*p)) { return -1; }
-            return atoi(p);
+            if (0 == *p || !isdigit(*p)) { return NO_TRACK; }
+            char* pLast;
+            double d (int(strtol(p, &pLast, 10)));
+            for (; 0 != *pLast && !isdigit(*pLast); ++pLast) {}
+            if (0 != *pLast)
+            {
+                d += atoi(pLast)/100000.0; // this sort of takes care of tracks in the format <total>/<track>
+            }
+            return d;
         }
 
         bool operator()(const Mp3HandlerTagData* p1, const Mp3HandlerTagData* p2) const
@@ -805,17 +924,19 @@ namespace
             // !!! this defines a "strict partial order" (irreflexive and transitive) iff the codes for digits are grouped together (as is the case with ASCII)
             const string& s1 (p1->getData(TagReader::TRACK_NUMBER));
             const string& s2 (p2->getData(TagReader::TRACK_NUMBER));
-            int n1 (getTrack(s1));
-            int n2 (getTrack(s2));
-            if (n1 >= 0 && n2 >= 0) { return n1 < n2; }
+            double d1 (getTrack(s1));
+            double d2 (getTrack(s2));
+            //if (d1 < NO_TRACK && d2 < NO_TRACK) { return d1 < d2; }
+            if (d1 != d2) { return d1 < d2; }
             return s1 < s2;
         }
     };
 }
-//ttt0 note on albums with no track number that they can use patterns
+
+
 
 void TagWriter::sortSongs() // sorts by track number; shows a warning if issues are detected (should be exactly one track number, from 1 to the track count)
-{ //ttt0 put last tracks with no track number
+{
     stable_sort(m_vpMp3HandlerTagData.begin(), m_vpMp3HandlerTagData.end(), SortByTrack());
     int n (cSize(m_vpMp3HandlerTagData));
     vector<pair<int, int> > v (n);
@@ -834,15 +955,13 @@ void TagWriter::sortSongs() // sorts by track number; shows a warning if issues
     m_bNonStandardTrackNo = false;
     for (int i = 0; i < n; ++i)
     {
-        if (SortByTrack::getTrack(m_vpMp3HandlerTagData[i]->getData(TagReader::TRACK_NUMBER)) != i + 1)
+        if (int(SortByTrack::getTrack(m_vpMp3HandlerTagData[i]->getData(TagReader::TRACK_NUMBER))) != i + 1)
         {
             m_bNonStandardTrackNo = true;
             if (m_pCommonData->m_bWarnOnNonSeqTracks && !m_bShowedNonSeqWarn)
             {
-                CursorOverrider crs (Qt::ArrowCursor);
-                QMessageBox::warning(m_pParentWnd, "Warning", "Track numbers are supposed to be consecutive numbers from 1 to the total number of tracks. This is not the case with the current album, which may lead to incorrect assignments when pasting information.");
                 m_bShowedNonSeqWarn = true;
-                //ttt1 when this is shown, the number of lines in the album grid is for the old album, but the text is from the new one
+                QTimer::singleShot(1, this, SLOT(onDelayedTrackSeqWarn()));
             }
             break;
         }
@@ -850,7 +969,11 @@ void TagWriter::sortSongs() // sorts by track number; shows a warning if issues
 
 }
 
-
+void TagWriter::onDelayedTrackSeqWarn()
+{
+    //CursorOverrider crs (Qt::ArrowCursor);
+    QMessageBox::warning(m_pParentWnd, "Warning", "Track numbers are supposed to be consecutive numbers from 1 to the total number of tracks. This is not the case with the current album, which may lead to incorrect assignments when pasting information.");
+}
 
 
 // returns 0 if there's no current handler
@@ -874,6 +997,16 @@ const Mp3HandlerTagData* TagWriter::getCrtMp3HandlerTagData() const
     return m_vpMp3HandlerTagData[m_nCurrentFile];
 }
 
+bool s_bToldAboutPatternsInCrtRun (false); // to limit to 1 per run the number of times the user is told about support
+
+
+void TagWriter::onDelayedChangeNotif()
+{
+    m_bWaitingChangeNotif = false;
+    QMessageBox::critical(m_pParentWnd, "Error", "Some files have been modified by an external tool after the last scan. You won't be able to save any changes to those files until you rescan them.");
+}
+
+
 
 //void TagWriter::reloadAll(string strCrt, ReloadOption eReloadOption/*, bool bKeepUnassgnImg*/)
 void TagWriter::reloadAll(string strCrt, bool bClearData, bool bClearAssgn)
@@ -885,8 +1018,8 @@ void TagWriter::reloadAll(string strCrt, bool bClearData, bool bClearAssgn)
     {
         strCrt = getCurrentName();
     }
-//ttt0 ??? q ? m_vpMp3HandlerTagData gets destroyed then prev values used?" x
-//ttt0 ??? ? push_back for m_vpMp3HandlerTagData ?
+//ttt2 ??? q ? m_vpMp3HandlerTagData gets destroyed then prev values used?" x
+//ttt2 ??? ? push_back for m_vpMp3HandlerTagData ?
     vector<Mp3HandlerTagData*> v; v.swap(m_vpMp3HandlerTagData);
     const deque<const Mp3Handler*>& vpHndl (m_pCommonData->getCrtAlbum());
     if (!v.empty() && cSize(v) != cSize(vpHndl))
@@ -907,6 +1040,11 @@ void TagWriter::reloadAll(string strCrt, bool bClearData, bool bClearAssgn)
         m_snUnassignedImages.clear();
     }
 
+    if (bClearAssgn)
+    {
+        m_bAutoVarArtists = true;
+    }
+
 
     m_vTagReaderInfo.clear();
 
@@ -924,12 +1062,15 @@ void TagWriter::reloadAll(string strCrt, bool bClearData, bool bClearAssgn)
     CB_ASSERT (v.empty() || cSize(v) == cSize(vpHndl));
 
     int n (cSize(vpHndl));
+    bool bFullReaderNotFound (false); // ID3V1 is not full, while the others are
 
     for (int i = 0; i < n; ++i)
     {
         const Mp3Handler* p (vpHndl[i]);
         std::map<std::string, int> m;
 
+        bool bTrackHasFullReader (false);
+
         const vector<DataStream*>& vStreams (p->getStreams());
         for (int i = 0, n = cSize(vStreams); i < n; ++i)
         {
@@ -939,14 +1080,19 @@ void TagWriter::reloadAll(string strCrt, bool bClearData, bool bClearAssgn)
                 sReaders.insert(make_pair(string(pRd->getName()), m[pRd->getName()]));
                 ++m[pRd->getName()];// = m[pRd->getName()] + 1;
 
+                if (pRd->getName() != Id3V1Stream::getClassDisplayName()) { bTrackHasFullReader = true; }
+
                 mReaderCount[pRd->getName()] = max(mReaderCount[pRd->getName()], m[pRd->getName()]);
 
                 if (bClearData)
                 { // add images
                     try
                     {
-                        ImageInfo img (pRd->getImage());
-                        m_imageColl.addImage(img); // doesn't matter if it's null or already exists; m_imageColl handles it correctly
+                        const vector<ImageInfo> vImg (pRd->getImages());
+                        for (int i = 0; i < cSize(vImg); ++i)
+                        {
+                            m_imageColl.addImage(vImg[i]); // doesn't matter if it's null or already exists; m_imageColl handles it correctly
+                        }
                     }
                     catch (const NotSupportedOp&)
                     {
@@ -954,12 +1100,18 @@ void TagWriter::reloadAll(string strCrt, bool bClearData, bool bClearAssgn)
                 }
             }
         }
+
+        if (!bTrackHasFullReader) { bFullReaderNotFound = true; }
     }
 
     for (int i = 0, n = cSize(m_vpTrackTextParsers); i < n; ++i)
     {
-        sReaders.insert(make_pair(string(TrackTextReader::getClassDisplayName()), i));
+        if (m_snActivePatterns.count(i) > 0)
+        {
+            sReaders.insert(make_pair(string(TrackTextReader::getClassDisplayName()), i));
+        }
     }
+
     mReaderCount[TrackTextReader::getClassDisplayName()] = cSize(m_vpTrackTextParsers);
 
     for (int i = 0, n = cSize(m_vAlbumInfo); i < n; ++i)
@@ -985,6 +1137,7 @@ void TagWriter::reloadAll(string strCrt, bool bClearData, bool bClearAssgn)
         }
     }
 
+    bool bRescanNeeded (false);
     { // process handlers in the order in which they appear in vpHndl;
         m_vpMp3HandlerTagData.resize(n);
         int nPastedSize (cSize(m_vstrPastedValues));
@@ -992,6 +1145,7 @@ void TagWriter::reloadAll(string strCrt, bool bClearData, bool bClearAssgn)
         {
             int k (m_vnMovedTo[i]);
             const Mp3Handler* p (vpHndl[i]);
+            bRescanNeeded = bRescanNeeded || p->needsReload(m_pCommonData->useFastSave());
             m_vpMp3HandlerTagData[k] = new Mp3HandlerTagData(this, p, k, i, k < nPastedSize ? m_vstrPastedValues[k] : "");
             //cout << "k=" << k << ", i=" << i << ", val=" << (k < nPastedSize ? m_vstrPastedValues[k] : "") << endl;
             if (!v.empty() && !bClearAssgn)
@@ -1010,8 +1164,41 @@ void TagWriter::reloadAll(string strCrt, bool bClearData, bool bClearAssgn)
                 }
             }
         }
+
+        adjustVarArtists();
     }
 
+    if (bRescanNeeded && !m_bWaitingChangeNotif)
+    {
+        m_bWaitingChangeNotif = true;
+        QTimer::singleShot(1, this, SLOT(onDelayedChangeNotif()));
+    }
+
+
+    m_bShouldShowPatternsNote = false;
+    if (!m_pCommonData->m_bToldAboutPatterns && !s_bToldAboutPatternsInCrtRun)
+    {
+        bool bIncompleteInfo (false); // turns true if a track is found that has empty artist, track number, or title; or if it only has ID3V1
+
+        if (!bFullReaderNotFound)
+        {
+            for (int i = 0; i < n; ++i)
+            {
+                if (m_vpMp3HandlerTagData[i]->getData(TagReader::TRACK_NUMBER).empty() || m_vpMp3HandlerTagData[i]->getData(TagReader::TITLE).empty() || m_vpMp3HandlerTagData[i]->getData(TagReader::ALBUM).empty() || m_vpMp3HandlerTagData[i]->getData(TagReader::ARTIST).empty())
+                {
+                    bIncompleteInfo = true;
+                    break;
+                }
+            }
+        }
+
+        if (bIncompleteInfo || bFullReaderNotFound)
+        {
+            m_bShouldShowPatternsNote = true;
+        }
+    }
+
+
     if (v.empty())
     {
         sortSongs();
@@ -1024,7 +1211,7 @@ void TagWriter::reloadAll(string strCrt, bool bClearData, bool bClearAssgn)
     { // add images from crt dir
         if (n > 0)
         { // scan crt dir for images //ttt2 this needs rewrite if an album means something else than a directory
-            CursorOverrider crs ();
+            CursorOverrider crs;
             string strDir (vpHndl[0]->getDir());
             FileSearcher fs (strDir);
 
@@ -1054,9 +1241,91 @@ void TagWriter::reloadAll(string strCrt, bool bClearData, bool bClearAssgn)
     emit albumChanged(/*DONT_CLEAR == eReloadOption*/);
     emit imagesChanged();
     setCrt(strCrt);
+    emit varArtistsUpdated(m_bVariousArtists);
 }
 
 
+void TagWriter::adjustVarArtists()
+{
+    if (!m_bAutoVarArtists) { return; } // !!! this gets called from reloadAll(); without the test it would lead to overriding whatever the user set
+
+    bool bPrev (m_bVariousArtists);
+    m_bVariousArtists = false;
+    int n (cSize(m_vpMp3HandlerTagData));
+
+    if (n > 1)
+    {
+        string strArtist (m_vpMp3HandlerTagData[0]->getData(TagReader::ARTIST));
+
+        for (int i = 1; i < n; ++i)
+        {
+            if (strArtist != m_vpMp3HandlerTagData[i]->getData(TagReader::ARTIST))
+            {
+                m_bVariousArtists = true;
+                break;
+            }
+        }
+    }
+
+    for (int i = 0; i < n; ++i)
+    {
+        m_vpMp3HandlerTagData[i]->adjustVarArtists(m_bVariousArtists);
+    }
+
+    if (bPrev != m_bVariousArtists)
+    {
+        emit varArtistsUpdated(m_bVariousArtists);
+    }
+}
+
+
+void TagWriter::toggleVarArtists()
+{
+    m_bAutoVarArtists = false;
+    m_bVariousArtists = !m_bVariousArtists;
+    string s (m_bVariousArtists ? TagReader::getVarArtistsValue() : "");
+
+    set<int> snSelSongs;
+    for (set<OrigValue>::iterator it = m_sSelOrigVal.begin(), end = m_sSelOrigVal.end(); it != end; ++it)
+    {
+        const OrigValue& o (*it);
+        if (o.m_nField == (int)TagReader::VARIOUS_ARTISTS)
+        {
+            snSelSongs.insert(o.m_nSong);
+        }
+    }
+
+    for (int i = 0; i < cSize(m_vpMp3HandlerTagData); ++i)
+    {
+        Mp3HandlerTagData* p (m_vpMp3HandlerTagData[i]);
+        if (Mp3HandlerTagData::ASSIGNED != p->getStatus(TagReader::VARIOUS_ARTISTS) || m_nCurrentFile == i || snSelSongs.count(i) > 0)
+        {
+            p->setData(TagReader::VARIOUS_ARTISTS, s);
+        }
+    }
+
+    emit albumChanged();
+    emit varArtistsUpdated(m_bVariousArtists);
+}
+
+
+void TagWriter::delayedAdjVarArtists()
+{
+    if (m_bDelayedAdjVarArtists) { return; }
+
+    m_bDelayedAdjVarArtists = true;
+    QTimer::singleShot(1, this, SLOT(onDelayedAdjVarArtists()));
+}
+
+void TagWriter::onDelayedAdjVarArtists()
+{
+    m_bDelayedAdjVarArtists = false;
+    adjustVarArtists();
+    emit albumChanged();
+}
+
+
+
 void TagWriter::setCrt(const std::string& strCrt)
 {
     m_nCurrentFile = 0;
@@ -1098,6 +1367,16 @@ void TagWriter::setCrt(int nCrt)
 // doesn't throw, but invalid patterns are discarded; it returns false if at least one pattern was discarded;
 bool TagWriter::updatePatterns(const std::vector<std::pair<std::string, int> >& v)
 {
+    set<string> sstrActive;
+    {
+        vector<string> v (getPatterns());
+        set<int> s (getActivePatterns());
+        for (set<int>::const_iterator it = s.begin(); it != s.end(); ++it)
+        {
+            sstrActive.insert(v[*it]);
+        }
+    }
+
     clearPtrContainer(m_vpTrackTextParsers);
     bool bRes (true);
 
@@ -1150,6 +1429,18 @@ bool TagWriter::updatePatterns(const std::vector<std::pair<std::string, int> >&
         vNew.swap(m_vSortedKnownTagReaders);
     }
 
+    {
+        vector<string> v (getPatterns());
+        m_snActivePatterns.clear();
+        for (int i = 0; i < cSize(v); ++i)
+        {
+            if (sstrActive.count(v[i]) > 0)
+            {
+                m_snActivePatterns.insert(i); //ttt2 not quite right: for 2 identical patterns of which one was selected before, both will be selected now;
+            }
+        }
+    }
+
     reloadAll("", DONT_CLEAR_DATA, DONT_CLEAR_ASSGN);
 
     return bRes;
@@ -1167,6 +1458,12 @@ vector<string> TagWriter::getPatterns() const
 }
 
 
+void TagWriter::setActivePatterns(const std::set<int>& s)
+{
+    m_snActivePatterns = s;
+    reloadAll("", DONT_CLEAR_DATA, DONT_CLEAR_ASSGN);
+}
+
 
 int TagWriter::getIndex(const ImageInfo& img) const
 {
@@ -1341,7 +1638,7 @@ void TagWriter::onEraseFileDelayed()
     if (inf.m_sstrFiles.size() > 1)
     {
         s = "these files?";
-        for (set<string>::iterator it = inf.m_sstrFiles.begin(); it != inf.m_sstrFiles.end(); ++it)
+        for (set<string>::const_iterator it = inf.m_sstrFiles.begin(); it != inf.m_sstrFiles.end(); ++it)
         {
             s += "\n" + convStr(toNativeSeparators(*it));
         }
@@ -1366,7 +1663,12 @@ void TagWriter::onEraseFileDelayed()
         if (bAssigned) { return; }
     }
 
-    for (set<string>::iterator it = inf.m_sstrFiles.begin(); it != inf.m_sstrFiles.end(); ++it)
+    if (!m_vAlbumInfo.empty() || !m_vstrPastedValues.empty())
+    {
+        if (0 != showMessage(m_pParentWnd, QMessageBox::Critical, 1, 1, "Warning", "Erasing image files triggers a full reload, which results in downloaded and pasted data being lost. Erase anyway?", "Erase", "Cancel")) { return; }
+    }
+
+    for (set<string>::const_iterator it = inf.m_sstrFiles.begin(); it != inf.m_sstrFiles.end(); ++it)
     {
         if (!QFile(convStr(*it)).remove())
         {
@@ -1473,7 +1775,7 @@ bool TagWriter::addImgFromFile(const QString& qs, bool bConsiderAssigned)
 
         int nSize ((int)f.size());
         QByteArray comprImg (f.read(nSize));
-        QPixmap pic;
+        QImage pic;
 
         if (pic.loadFromData(comprImg))
         {
@@ -1485,11 +1787,11 @@ bool TagWriter::addImgFromFile(const QString& qs, bool bConsiderAssigned)
             if (nSize <= ImageInfo::MAX_IMAGE_SIZE)
             {
                 nWidth = pic.width(); nHeight = pic.height();
-                eCompr = qs.endsWith(".png", Qt::CaseInsensitive) ? ImageInfo::PNG : ImageInfo::JPG; //ttt2 add more cases if supporting more image tipes
+                eCompr = qs.endsWith(".png", Qt::CaseInsensitive) ? ImageInfo::PNG : ImageInfo::JPG; //ttt2 add more cases if supporting more image types
             }
             else
             {
-                QPixmap scaledImg;
+                QImage scaledImg;
                 ImageInfo::compress(pic, scaledImg, comprImg);
                 nWidth = scaledImg.width(); nHeight = scaledImg.height();
                 eCompr = ImageInfo::JPG;
@@ -1526,17 +1828,17 @@ void TagWriter::paste()
     CursorOverrider crs;
 
     QClipboard* pClp (QApplication::clipboard());
-    QPixmap pic (pClp->pixmap()); //ttt1 this leads in many cases to recompression; see how to avoid it; (probably ask the clipboard for other formats)
+    QPixmap pic (pClp->pixmap()); //ttt2 this leads in many cases to recompression; see how to avoid it; (probably ask the clipboard for other formats)
     if (!pic.isNull())
     {
-        ImageInfo img (-1, ImageInfo::OK, pic);
-        //ttt1 only reason to change image widget is because images were loaded, so perhaps adjust signals; (keep in mind first time, though)
+        ImageInfo img (-1, ImageInfo::OK, pic.toImage());
+        //ttt2 only reason to change image widget is because images were loaded, so perhaps adjust signals; (keep in mind first time, though)
         //emit imagesChanged();
         addImage(img, CONSIDER_UNASSIGNED);
         return;
     }
 
-    //ttt1 text file name, including "http://", "ftp://", "file://" (see KIO::NetAccess::download() at http://developer.kde.org/documentation/books/kde-2.0-development/ch07lev1sec5.html )
+    //ttt2 text file name, including "http://", "ftp://", "file://" (see KIO::NetAccess::download() at http://developer.kde.org/documentation/books/kde-2.0-development/ch07lev1sec5.html )
     QString qs (pClp->text());
 
     if (qs.isEmpty())
@@ -1548,7 +1850,7 @@ void TagWriter::paste()
         for (int i = 0; i < lst.size(); ++i)
         {
             QString qs1 (lst[i].toString());
-            //qDebug("#%s#", qs1.toUtf8().data());
+            //qDebug("#%s#", qs1.toUtf8().constData());
             if (!qs1.isEmpty())
             {
                 if (qs.isEmpty())
@@ -1594,10 +1896,10 @@ void TagWriter::paste()
 #endif
 
 #ifndef WIN32
-            if (qs.startsWith(getPathSep())) // ttt1 see if it makes sense to open files without full name
+            if (qs.startsWith(getPathSep())) // ttt2 see if it makes sense to open files without full name
 #else
             qs = fromNativeSeparators(qs);
-            //qDebug("qs=%s", qs.toUtf8().data());
+            //qDebug("qs=%s", qs.toUtf8().constData());
             if (qs.size() > 7 && qs[0].isLetter() && qs[1] == ':' && qs[2] == getPathSep())
 #endif
             {
